/**
 * @file uldmanager.h
 * @brief The interface for the <code>ULDMIterator</code> class.
 * @author Jeff Ondich
 */

#ifndef ULDMITERATOR_H
#define ULDMITERATOR_H

// This has to be here, and...
#ifdef UL_REGEX

#ifdef WIN32
#    ifndef UL_WINDOWSMOBILE
#        include "boost/regex.h"
#    endif
#else
#    include <sys/types.h>
#    include <regex.h>
#endif

#include "ultokenizer.h"
#endif

// ...this has to be separate, so the users of the APIs can use ulregex.h to get their
// hands on the ULDMRegexOptions struct, without being required to load boost.
#include "ulregex.h"

#include "ulditerator.h"
#include "ulstringpool.h"
#include "ulrendereroptions.h"
#include "ultermidentifier.h"

/**
 * @class ULDMIterator
 * @brief The class that combines a collection of <code>ULDIterator</code> objects
 *   pointing into distinct <code>ULD</code> modules with identical languages.
 *   <code>ULDMIterator</code> does the runtime work of merging the data contained
 *   in separate data modules.
 *
 * The member functions of <code>ULDMIterator</code> maintain the following
 * invariant:
 *
 * For all i < mIteratorList.size()
 *
 *      *mIteratorList[0] <= *mIteratorList[i]
 * and
 *      *(--mIteratorList[i]) <= *(mIteratorList[0])
 *
 * This invariant allows us to consider mIteratorList[0] to be the current
 * pointer, and to efficiently execute operator--.  The invariant is enforced
 * by <code>ULDMIterator::adjust</code>, and checked by
 * <code>ULDMIterator::checkIntegrity</code>.
 */

class UL_API ULDMIterator
{
    friend class ULDManager;
    friend class ULTermIdentifier;

public:
                            ULDMIterator();
                            ULDMIterator( const ULDMIterator& it );
                            ~ULDMIterator();

    void                    clear();
    void                    addIterator( const ULDIterator& it );

    bool                    operator==( const ULDMIterator& it ) const;
    ULDMIterator&           operator=( const ULDMIterator& it );
    const ULTerm&           operator*() const;
    ULDMIterator&           operator++();
    ULDMIterator            operator++( int );
    ULDMIterator&           operator--();
    ULDMIterator            operator--( int );
    
    void                    stepForward();
    void                    stepBack();

    bool                    isAtEnd() const;
    bool                    isAtBeginning() const;
    bool                    isAtModifiableTerm() const;
    bool                    isAtFirstTermOfHeadword() const;
    bool                    isAtLastTermOfHeadword() const;

    uint32                  hash( uint32 tableSize ) const;

    ulstring                toXHTML( const ULStringPool &rendererStrings, const ULRendererOptions& options ) const;
    ulstring                toIPadXHTML( const ULStringPool &rendererStrings, const ULRendererOptions& options, const ulstring verbTable ) const;
    ulstring                toCollinsXHTML( const ULStringPool &rendererStrings, const ULRendererOptions& options ) const;
	ulstring				toCollinsIPadXHTML( const ULStringPool &rendererStrings, const ULRendererOptions& options, const ulstring verbTable ) const;
    ulstring                toVoxXHTML( const ULStringPool &rendererStrings, const ULRendererOptions& options ) const;
    ulstring                toVoxIPadXHTML( const ULStringPool &rendererStrings, const ULRendererOptions& options, const ulstring verbTable ) const;
    ulstring                getTTSPath() const;

    ULD                     *getULD();
    const ULD               *getULD() const;
    bool                    isDefinitionModifiable( int32 definitionIndex );
    bool                    deleteDefinition( int32 definitionIndex );
    void                    move( uint32 positionNumerator, uint32 positionDenominator );

    uint32                  getMaxPosition() const;
    uint32                  getMinPosition() const;
    uint32                  getPosition() const;
    uint32                  getTermOffset() const;

    uint32                  getProduct() const;
    ULLanguage              getSearchLanguage() const;
    ULLanguage              getTranslationLanguage() const;

    bool                    checkIntegrity();

    void                    getTermIdentifier( ULTermIdentifier& identifier ) const;

private:
    ULDManager              *mULDManager;
    ULTerm                  mTerm;
    bool                    mTermDirty;
    ULList<ULDIterator>     mIteratorList;
    bool                    mUseRegexSearch;
    bool                    mIsRegexMatchInMainText;

    void                    adjust();
    void                    synchronize();

private:
    static void             startV7Feature( ulstring& s, uint32 feature );
    static void             endV7Feature( ulstring& s, uint32 feature );
    static void             startVoxV7Feature( ulstring& s, uint32 feature );
    static void             endVoxV7Feature( ulstring& s, uint32 feature );
    static void             startCollinsV7Feature( ulstring& s, uint32 feature );
    static void             endCollinsV7Feature( ulstring& s, uint32 feature );
    static bool             isCollinsGenericFeature( uint32 feature );
    static ulstring         fixCollinsInflection( const ulstring& inflectionText, bool isTermInflection );
    static void             formatV7FeatureText( ulstring& s, uint32 feature, bool addWordMarks, const ulstring& startWordMark, const ulstring& endWordMark );
    static void             formatV7FeatureText( ulstring& s, uint32 feature );
    static void             formatV7FeatureTextAfterWordMarkAdditions( ulstring& s, uint32 feature );
    static void             finishCollinsFormatting( ulstring& s, int nDefinitionNumbers );

#ifdef UL_REGEX
private:
    ULDMRegexOptions        mOptions;
    int                     mCFlags;
    regex_t                 mCompiledRegExp;
    regmatch_t              mMatch;
    bool                    mRestrictedRange;
    ulstring                mSearchString;
    ULTokenizer             *mTokenizer;

public:
    bool                    initRegexSearch( const ulstring& searchString,
                                             ULLanguage lang,
                                             const ULDMRegexOptions& options,
                                             ULTokenizer *tokenizer );

    bool                    usingRegex() const { return mUseRegexSearch; }
    bool                    isRegexMatch();
#endif
};


#endif
