/**
 * @file ulstring.h
 * @brief The interface for the <code>ulstring</code>,
 *   <code>ulstring_iterator</code>, and <code>ulstring_reverse_iterator</code>
 *   classes.
 * @author Jeff Ondich
 */

#ifndef ULSTRING_H
#define ULSTRING_H

#include "ultypes.h"

#ifdef UL_SYMBIAN
#include <stdarg_e.h>
#define va_list __e32_va_list
#else
#include <cstdarg>
#endif

// Some g++ configurations are using macros for toupper and tolower,
// which causes trouble for ulstring::toupper and ulstring::tolower.
#undef tolower
#undef toupper

/**
 * Encoding constants for use with <code>ulstring</code>.
 */
enum
{
    kULSFirstEncoding = 0,
    kULSUTF8 = kULSFirstEncoding,
    kULSUCS2,
    kULSUCS4,
    kULSLatin1,
    kULSByteString = kULSLatin1,
    kULSLastEncoding
};

/**
 * Byte-order constants for use with <code>ulstring</code>.
 */
enum
{
    kULSBigEndian = 0,
    kULSLittleEndian
};

/**
 * OR-able options for the main comparison function.
 */
enum
{
    kULSCompareOptionIgnoreCase = 1,
    kULSCompareOptionIgnoreAccents = (1 << 1),
    kULSCompareOptionIgnoreSeparators = (1 << 2),
    kULSCompareOptionAlphaOnly = (1 << 3),
    kULSCompareOptionMatchPrefix = (1 << 4)
};


class UL_API ulstring_iterator;
class UL_API ulstring_reverse_iterator;

#define ULSTRING_ITERATOR_IS_AT_END (mCharIndex == -1  &&  mByteIndex == -1)

#define ULSTRING_ITERATOR_INVARIANT (ULSTRING_ITERATOR_IS_AT_END  ||\
                                     (mString != 0 && mCharIndex >= 0 && mCharIndex < mString->mCharLength\
                                         && mByteIndex >= 0 && mByteIndex < mString->mByteCount))

#define ULSTRING_INVARIANT (mBuffer != 0 && mCapacity > 0 && mByteCount + sizeOfNull(mEncoding) <= mCapacity\
                                && mCharLength <= mByteCount && mCharLength >= 0)

// Utility functions.
int sizeOfNull( int encoding );
int putULChar( char *p, ulchar ch, int encoding, int byteOrder );


/**
 * @class ulstring
 * @brief A string class for use in the Ultralingua API.
 *
 * Motivations for reinventing the wheel instead of just using
 * the STL string:
 *
 * <ol>
 * <li>You can't use inheritance on Palm OS if you want the code to be
 *   usable with launch codes that don't make globals available.</li>
 * <li>We wanted to encapsulate conversion between encodings, with control over
 *   the source code.</li>
 * </ol>
 *
 * Principles:
 *
 * <ul>
 * <li>All char * parameters sent to member functions are assumed to be in
 *   the byte order and encoding of the string being acted upon.  Thus,
 *   if you want to send the literal "emu" to a <code>ulstring</code> that is
 *   currently set to big-endian and UCS-2, you could use "\0e\0m\0u\0\0".</li>
 * <li>No inheritance allowed.</li>
 * <li>Individual characters are accessed using iterators, since different
 *   encodings will have different character lengths.  Beware of the linear-time
 *   performance of <code>operator[]</code> and <code>setAt</code>.</li>
 * <li>Data storage is array-based, not list-based, so operations you would
 *   expect to take linear time in STL vectors will also take linear time
 *   here.  Both <code>size</code> (number of bytes) and <code>length</code>
 *   (number of characters) are constant-time.</li>
 * </ul>
 */
class UL_API ulstring
{
    friend class ulstring_iterator;
    friend class ulstring_reverse_iterator;

private:
    char            *mBuffer;
    int             mCapacity;

    int             mEncoding;
    int             mByteOrder;

    // Number of bytes/characters of meaningful data, NOT including null termination.
    int             mByteCount;
    int             mCharLength;

#ifdef UL_PALMOS
    enum { kMinCapacity = 16 };
#else
    enum { kMinCapacity = 32 };
#endif

    int             bytesRequired( int newEncoding );
    bool            reallocate( int nBytesNeeded, bool copyData );

public:
                    ulstring( int encoding=kULSUTF8, int byteOrder=kULSBigEndian );
                    ulstring( const char *s, int encoding=kULSUTF8, int byteOrder=kULSBigEndian );
                    ulstring( const ulstring& s );

                    ~ulstring();

    int             getEncoding() const;
    void            setEncoding( int newEncoding );
    int             getByteOrder() const;
    void            setByteOrder( int newByteOrder );

    ulstring&       operator=( const ulstring& s );
    ulstring&       operator=( const char *s );
    ulstring&       operator=( ulchar ch );

    ulstring&       operator+=( const ulstring& s );
    ulstring&       operator+=( const char *s );
    ulstring&       operator+=( ulchar ch );

    ulchar          operator[]( int n ) const;

    int             compare( const char *s ) const;
    int             compare( const ulstring& s, uint32 options, ULLanguage lang=kULNeutralLanguage ) const;
    int             compare( const ulstring& s, ULLanguage lang=kULNeutralLanguage ) const;
    int             compare( const ulstring& s, bool ignoreCase, bool ignoreAccents, ULLanguage lang=kULNeutralLanguage ) const;

    ulstring_iterator           begin() const;
    ulstring_iterator           end() const;
    ulstring_reverse_iterator   rbegin() const;
    ulstring_reverse_iterator   rend() const;

    int             length() const;
    int             size() const;
    int             sizeWithNull() const;

    bool            empty() const;
    void            clear();

    void            lower();
    void            upper();
    void            tolower();
    void            toupper();
    void            tobase( ULLanguage language=kULNeutralLanguage );
    void            expand();
    void            clean( ULLanguage language, bool toupper=true );
    void            urlencode();
    void            xmlEscape();
    void            strip();
    void            stripNonAlpha();
    void            rtrim();
    void            ltrim();
    void            trim();
    int             toint();
    bool            isdigit() const;

    void            appendFormat( const char* formatString, ... );

    void            insert( ulstring_iterator it, const ulstring& s );
    void            insert( ulstring_iterator it, ulchar ch );
    void            insert( ulstring_iterator it, const char *s );
    void            insert( ulstring_reverse_iterator it, const ulstring& s );
    void            insert( ulstring_reverse_iterator it, ulchar ch );
    void            insert( ulstring_reverse_iterator it, const char *s );

    ulchar          chop();

    void            erase( ulstring_iterator it );
    void            erase( ulstring_iterator it, int n );
    void            erase( int start, int n );
    void            erase( ulstring_reverse_iterator it );
    void            erase( ulstring_reverse_iterator it, int n );

    void            replace( ulstring_iterator start, ulstring_iterator stop, const ulstring& s );
    void            replace( ulstring_iterator start, ulstring_iterator stop, ulchar ch );
    void            replace( ulstring_iterator start, ulstring_iterator stop, const char *s );
    void            replace( ulstring_iterator start, int n, const ulstring& s );
    void            replace( ulstring_iterator start, int n, ulchar ch );
    void            replace( ulstring_iterator start, int n, const char *s );
    void            replace( int start, int n, const ulstring& s );
    void            replace( int start, int n, ulchar ch );
    void            replace( int start, int n, const char *s );
    void            replace( ulstring_reverse_iterator start, ulstring_reverse_iterator stop, const ulstring& s );
    void            replace( ulstring_reverse_iterator start, ulstring_reverse_iterator stop, ulchar ch );
    void            replace( ulstring_reverse_iterator start, ulstring_reverse_iterator stop, const char *s );
    void            replace( const ulstring& toBeReplaced, const ulstring& replacement );
    void            replace( ulchar toBeReplaced, const ulstring& replacement );

    void            setAt( int n, ulchar ch );
    void            setAt( ulstring_iterator it, ulchar ch );
 
    ulstring        substr( ulstring_iterator start, ulstring_iterator stop ) const;
    ulstring        substr( int start, int nChars=-1 ) const;

    ulstring_iterator        find( ulchar ch ) const;
    ulstring_iterator        find( const ulstring& s ) const;
    ulstring_iterator        find( const char *s ) const;
    ulstring_iterator        find( const ulstring_iterator& start, ulchar ch ) const;
    ulstring_iterator        find( const ulstring_iterator& start, const ulstring& s ) const;
    ulstring_iterator        find( const ulstring_iterator& start, const char *s ) const;
    ulstring_iterator        rfind( ulchar ch ) const;
    ulstring_iterator        rfind( const ulstring& s ) const;
    ulstring_iterator        rfind( const char *s ) const;
    bool                     matchPrefix( const ulstring& s, bool ignoreCase, bool ignoreAccents ) const;

    void            swap( ulstring& s );
    const char *    c_str() const;
    void            append( const char *buffer, uint32 nBytes );

    int             getIntValue() const;
    uint32          hash( uint32 tableSize ) const;

    static void     unitTest();

#if defined(UL_ANSI) && !defined(UL_NOSTL) 
    void            dump( ostream& out );
    static void        putChar( ostream& out, ulchar ch, int encoding, int byteOrder );
#endif

#ifdef UL_PALMOS
    void            convertToPalmEncoding();
#endif
};

UL_API ulstring operator+( const ulstring& a, const ulstring& b );
UL_API ulstring operator+( const ulstring& a, const char *s );
UL_API ulstring operator+( const ulstring& a, const char ch );
UL_API ulstring operator+( const ulstring& a, uint32 k );

UL_API bool operator<( const ulstring& a, const char *s );
UL_API bool operator<=( const ulstring& a, const char *s );
UL_API bool operator>( const ulstring& a, const char *s );
UL_API bool operator>=( const ulstring& a, const char *s );
UL_API bool operator==( const ulstring& a, const char *s );
UL_API bool operator!=( const ulstring& a, const char *s );

UL_API bool operator<( const ulstring& a, const ulstring& b );
UL_API bool operator<=( const ulstring& a, const ulstring& b );
UL_API bool operator>( const ulstring& a, const ulstring& b );
UL_API bool operator>=( const ulstring& a, const ulstring& b );
UL_API bool operator==( const ulstring& a, const ulstring& b );
UL_API bool operator!=( const ulstring& a, const ulstring& b );

#if defined(UL_ANSI) && !defined(UL_NOSTL) 
UL_API ostream& operator<<( ostream& out, const ulstring& s );
UL_API istream& operator>>( istream& in, ulstring& s );
UL_API istream& getline( istream& in, ulstring& s, ulchar delimiter='\n' );
UL_API istream& getstring( istream& in, ulstring& s );
#endif


/**
 * @class ulstring_iterator
 * @brief An iterator that moves forward (via operator++) through a
 *   <code>ulstring</code> object.
 */
class UL_API ulstring_iterator
{
    friend class ulstring_reverse_iterator;

private:
    int                 mByteIndex;
    int                 mCharIndex;
    const ulstring      *mString;

public:
            ulstring_iterator() : mByteIndex(-1), mCharIndex(-1), mString(0) {}
            ulstring_iterator( const ulstring_iterator& it );
            ulstring_iterator( const ulstring *s );
            ~ulstring_iterator();

    ulstring_iterator& operator=( const ulstring_iterator& s );

    bool    operator==( const ulstring_iterator& it ) const;
    bool    operator!=( const ulstring_iterator& it ) const;
    bool    operator<( const ulstring_iterator& it ) const { return (mByteIndex < it.mByteIndex  &&  mString == it.mString); }

    int     getCharIndex() const;
    int     getByteIndex() const;

    const ulstring *getString() const;

    bool    isAtEnd() const;
    bool    isAtBeginning() const;
    void    moveToEnd();
    void    moveToBeginning();

    void    operator++();
    void    operator++(int);
    void    operator--();
    void    operator--(int);

    void    operator+=( int nChars );
    void    operator-=( int nChars );

    ulchar  operator*() const;
};


/**
 * @class ulstring_reverse_iterator
 * @brief An iterator that moves backwards (via operator++) through a
 *   <code>ulstring</code> object.
 */
class UL_API ulstring_reverse_iterator
{
    friend class ulstring_iterator;

private:
    int             mByteIndex;
    int             mCharIndex;
    const ulstring  *mString;

public:
            ulstring_reverse_iterator() : mByteIndex(0), mCharIndex(0), mString(0) {}

            ulstring_reverse_iterator( const ulstring_reverse_iterator& it )
                : mByteIndex(it.mByteIndex), mCharIndex(it.mCharIndex), mString(it.mString) {}

            ulstring_reverse_iterator( const ulstring *s );

            ~ulstring_reverse_iterator() {}

    ulstring_reverse_iterator& operator=( const ulstring_reverse_iterator& it );

            operator ulstring_iterator() const;

    bool    operator==( const ulstring_reverse_iterator& s ) const;
    bool    operator!=( const ulstring_reverse_iterator& s ) const { return !(*this == s); }
    bool    operator<( const ulstring_reverse_iterator& s ) const;

    int     getCharIndex() const { return mCharIndex; }
    int     getByteIndex() const { return mByteIndex; }

    const ulstring *getString() const { return mString; }

    bool    isAtEnd() const { return mString == 0  ||  mCharIndex == -1; }
    bool    isAtBeginning() const { return mString == 0  ||  mCharIndex >= mString->mCharLength - 1; }

    void    moveToEnd() { mCharIndex = mByteIndex = -1; }
    void    moveToBeginning();

    void    operator++();
    void    operator++(int);
    void    operator--();
    void    operator--(int);

    void    operator+=( int nChars );
    void    operator-=( int nChars );

    ulchar  operator*() const;
};


#endif
