///////////////////////////////////////////////////////////////////
//
//  ulcontainers.h
//
//  A few simple templated containers.  Nothing special, except
//  unlike the STL classes, these are inheritance-free, and thus
//  usable throughout Palm OS code.
//
///////////////////////////////////////////////////////////////////

#ifndef ULCONTAINERS_H
#define ULCONTAINERS_H

#include "ultypes.h"

////////////////////////////////////////////////////////////
// ULPair
////////////////////////////////////////////////////////////

template <class T, class U>
struct UL_API ULPair
{
    T                   mFirst;
    U                   mSecond;

                        ULPair();
                        ULPair( T first, U second );
                        ULPair( const T& first, const U& second );
                        ULPair( const ULPair<T,U>& p );

    ULPair<T,U>&        operator=( const ULPair<T,U>& p );

    bool                operator==( const ULPair<T,U>& p ) const;
    bool                operator<( const ULPair<T,U>& p ) const;
    bool                operator!=( const ULPair<T,U>& p ) const;
};


template <class T, class U>
ULPair<T,U>::ULPair()
{
}


template <class T, class U>
ULPair<T,U>::ULPair( T first, U second )
{
    mFirst = first;
    mSecond = second;
}


template <class T, class U>
ULPair<T,U>::ULPair( const T& first, const U& second )
{
    mFirst = first;
    mSecond = second;
}


template <class T, class U>
ULPair<T,U>::ULPair( const ULPair<T,U>& p )
{
    *this = p;
}


template <class T, class U>
ULPair<T,U>& ULPair<T,U>::operator=( const ULPair<T,U>& p )
{
    if( this != &p )
    {
        mFirst = p.mFirst;
        mSecond = p.mSecond;
    }

    return *this;
}


template <class T, class U>
bool ULPair<T,U>::operator==( const ULPair<T,U>& p ) const
{
    return (mFirst == p.mFirst  &&  mSecond == p.mSecond);
}


template <class T, class U>
bool ULPair<T,U>::operator!=( const ULPair<T,U>& p ) const
{
    return !(*this == p);
}


template <class T, class U>
bool ULPair<T,U>::operator<( const ULPair<T,U>& p ) const
{
    if( !(mFirst == p.mFirst) )
        return mFirst < p.mFirst;

    return mSecond < p.mSecond;
}



////////////////////////////////////////////////////////////
// ULListNode
////////////////////////////////////////////////////////////

template <class T>
struct ULListNode
{
    T                   mData;
    ULListNode          *mNext;
    ULListNode          *mPrevious;

                        ULListNode();
                        ULListNode( const T& data );
                        ULListNode( const ULListNode<T>& node );

    ULListNode<T>&      operator=( const ULListNode<T>& node );
};


template <class T>
ULListNode<T>::ULListNode()
{
    mNext = 0;
    mPrevious = 0;
}


template <class T>
ULListNode<T>::ULListNode( const T& data )
{
    mNext = 0;
    mPrevious = 0;
    mData = data;
}


template <class T>
ULListNode<T>::ULListNode( const ULListNode<T>& node )
{
    *this = node;
}


template <class T>
ULListNode<T>& ULListNode<T>::operator=( const ULListNode<T>& node )
{
    if( this != &node )
    {
        mData = node.mData;
        mNext = node.mNext;
        mPrevious = node.mPrevious;
    }

    return *this;
}


////////////////////////////////////////////////////////////
// ULListIterator
////////////////////////////////////////////////////////////

template <class T>
class ULList;

template <class T>
class UL_API ULListIterator
{
    friend class ULList<T>;

private:
    ULList<T>           *mList;
    ULListNode<T>       *mCurrent;

public:
                        ULListIterator();
                        ULListIterator( const ULListIterator<T>& it );

    void                clear();

    ULListIterator<T>&  operator=( const ULListIterator<T>& it );

    bool                operator==( const ULListIterator<T>& it );
    bool                operator!=( const ULListIterator<T>& it );

    ULListIterator<T>&  operator++();
    ULListIterator<T>   operator++( int );
    ULListIterator<T>&  operator--();
    ULListIterator<T>   operator--( int );

    T&                  operator*();

    bool                isAtEnd() const;
    bool                isAtBeginning() const;
};


template <class T>
ULListIterator<T>::ULListIterator()
{
    mCurrent = 0;
    mList = 0;
}


template <class T>
ULListIterator<T>::ULListIterator( const ULListIterator<T>& it )
{
    *this = it;
}


template <class T>
void ULListIterator<T>::clear()
{
    mCurrent = 0;
    mList = 0;
}


template <class T>
ULListIterator<T>&  ULListIterator<T>::operator=( const ULListIterator<T>& it )
{
    if( this != &it )
    {
        mCurrent = it.mCurrent;
        mList = it.mList;
    }

    return *this;
}

template <class T>
bool ULListIterator<T>::isAtEnd() const
{
    return (mCurrent == 0);
}


template <class T>
bool ULListIterator<T>::isAtBeginning() const
{
    return (mList == 0  ||  mCurrent == mList->mFront);
}


template <class T>
bool ULListIterator<T>::operator==( const ULListIterator<T>& it )
{
    return mCurrent == it.mCurrent  &&  mList == it.mList;
}


template <class T>
bool ULListIterator<T>::operator!=( const ULListIterator<T>& it )
{
    return !(*this == it);
}


template <class T>
ULListIterator<T>&  ULListIterator<T>::operator++()
{
    if( mCurrent != 0 )
        mCurrent = mCurrent->mNext;

    return *this;
}


template <class T>
ULListIterator<T>  ULListIterator<T>::operator++( int )
{
    ULListIterator<T> tmp = *this;
    ++(*this);
    return tmp;
}


template <class T>
ULListIterator<T>&  ULListIterator<T>::operator--()
{
    UL_ASSERT( mList != 0 );

    if( mCurrent == mList->mFront )
    {
    }

    else if( mCurrent != 0 )
        mCurrent = mCurrent->mPrevious;

    else if( mList != 0 )
        mCurrent = mList->mBack;

    return *this;
}


template <class T>
ULListIterator<T>  ULListIterator<T>::operator--( int )
{
    ULListIterator<T> tmp = *this;
    --(*this);
    return tmp;
}


template <class T>
T& ULListIterator<T>::operator*()
{
    UL_ASSERT( mCurrent != 0 );
    return mCurrent->mData;
}


////////////////////////////////////////////////////////////
// ULListConstIterator
////////////////////////////////////////////////////////////

template <class T>
class ULList;

template <class T>
class UL_API ULListConstIterator
{
    friend class ULList<T>;

private:
    const ULList<T>          *mList;
    const ULListNode<T>      *mCurrent;

public:
                             ULListConstIterator();
                             ULListConstIterator( const ULListConstIterator<T>& it );

    void                     clear();

    ULListConstIterator<T>&  operator=( const ULListConstIterator<T>& it );

    bool                     operator==( const ULListConstIterator<T>& it );
    bool                     operator!=( const ULListConstIterator<T>& it );

    ULListConstIterator<T>&  operator++();
    ULListConstIterator<T>   operator++( int );
    ULListConstIterator<T>&  operator--();
    ULListConstIterator<T>   operator--( int );

    const T&                 operator*() const;                   

    bool                     isAtEnd() const;
    bool                     isAtBeginning() const;
};


template <class T>
ULListConstIterator<T>::ULListConstIterator()
{
    mCurrent = 0;
    mList = 0;
}


template <class T>
ULListConstIterator<T>::ULListConstIterator( const ULListConstIterator<T>& it )
{
    *this = it;
}


template <class T>
void ULListConstIterator<T>::clear()
{
    mCurrent = 0;
    mList = 0;
}


template <class T>
ULListConstIterator<T>&  ULListConstIterator<T>::operator=( const ULListConstIterator<T>& it )
{
    if( this != &it )
    {
        mCurrent = it.mCurrent;
        mList = it.mList;
    }

    return *this;
}


template <class T>
bool ULListConstIterator<T>::isAtEnd() const
{
    return (mCurrent == 0);
}


template <class T>
bool ULListConstIterator<T>::isAtBeginning() const
{
    return (mList == 0  ||  mCurrent == mList->mFront);
}


template <class T>
bool ULListConstIterator<T>::operator==( const ULListConstIterator<T>& it )
{
    return mCurrent == it.mCurrent  &&  mList == it.mList;
}


template <class T>
bool ULListConstIterator<T>::operator!=( const ULListConstIterator<T>& it )
{
    return !(*this == it);
}


template <class T>
ULListConstIterator<T>&  ULListConstIterator<T>::operator++()
{
    if( mCurrent != 0 )
        mCurrent = mCurrent->mNext;

    return *this;
}


template <class T>
ULListConstIterator<T>  ULListConstIterator<T>::operator++( int )
{
    ULListConstIterator<T> tmp = *this;
    ++(*this);
    return tmp;
}


template <class T>
ULListConstIterator<T>&  ULListConstIterator<T>::operator--()
{
    UL_ASSERT( mList != 0 );

    if( mCurrent == mList->mFront )
    {
    }
    
    else if( mCurrent != 0 )
        mCurrent = mCurrent->mPrevious;

    else if( mList != 0 )
        mCurrent = mList->mBack;

    return *this;
}


template <class T>
ULListConstIterator<T>  ULListConstIterator<T>::operator--( int )
{
    ULListConstIterator<T> tmp = *this;
    --(*this);
    return tmp;
}


template <class T>
const T& ULListConstIterator<T>::operator*() const
{
    UL_ASSERT( mCurrent != 0 );
    return mCurrent->mData;
}


////////////////////////////////////////////////////////////
// ULList
////////////////////////////////////////////////////////////

template <class T>
class UL_API ULList
{
    friend class ULListIterator<T>;
    friend class ULListConstIterator<T>;

private:
    ULListNode<T>       *mFront;
    ULListNode<T>       *mBack;

    void                Destroy();

public:
                        ULList();
                        ULList( const ULList<T>& list );
                        ~ULList();

    ULList<T>&          operator=( const ULList<T>& list );

    T&                  operator[]( uint32 n ) const;

    void                clear() { Destroy();  mFront = mBack = 0; }
    
    // Find a node whose data matches the specified data.
    ULListIterator<T>   find( const T& data );

    // Remove the nth node.
    void                erase( uint32 n );
    // Remove the node pointed to by it.
    void                erase( ULListIterator<T>& it );
    // Remove all the nodes starting at start, and ending just before
    // stop.  (That is, the node pointed to by stop will remain in the
    // list.)
    void                erase( ULListIterator<T>& start, ULListIterator<T>& stop );

    bool                empty() const { return mFront == 0; }
    bool                hasOneOrFewerNodes() const { return mFront == mBack; }
    uint32              size() const;

    void                push_front( const T& data );
    void                push_back( const T& data );
    void                add( const T& data );  // Does push_back only if data is not already in the list
    void                insert( ULListIterator<T>& it, const T& data ); // inserts data at specified location
    void                insert( const T& data ); // inserts so as to keep list in increasing order, based on T::operator<

    // Stack operations (in conjunction with push_back).
    void                pop();
    T&                  top();

    ULListIterator<T>       begin();
    ULListConstIterator<T>  cbegin() const;
    ULListIterator<T>       end();
    ULListConstIterator<T>  cend() const;
    ULListIterator<T>       back();
    ULListConstIterator<T>  cback() const;

    // Node-based operations to allow rearrangement of the list
    // without lots of copying when T is large.
    ULListNode<T>       *removeNode( ULListIterator<T>& it );
    void                insertNode( ULListIterator<T>& it, ULListNode<T> *node );
    ULListNode<T>       *removeFrontNode(); // convenience function

    void                sort(int (*func)(T&, T&, void *), void *context);
    // the function in sort() takes two objects and a pointer to the
    // context that was passed in; it returns -1, 0, or 1 to indicate ordering
    void                mergeSort(int (*func)(T&, T&, void *), void *context);
    void                removeDuplicates();
};


template <class T>
ULList<T>::ULList()
{
    mFront = 0;
    mBack = 0;
}

template <class T>
ULList<T>::ULList( const ULList<T>& list )
{
    mFront = 0;
    mBack = 0;
    *this = list;
}


template <class T>
ULList<T>::~ULList()
{
    Destroy();
}


template <class T>
void ULList<T>::Destroy()
{
    ULListNode<T> *tmp;
    ULListNode<T> *current = mFront;

    while( current != 0 )
    {
        tmp = current;
        current = current->mNext;
        delete tmp;
    }
}


template <class T>
ULList<T>& ULList<T>::operator=( const ULList<T>& list )
{
    if( this == &list )
        return *this;

    ULListNode<T> *current, *tmp;

    Destroy();
    mFront = mBack = 0;
    for( current=list.mFront; current != 0; current = current->mNext )
    {
        tmp = new ULListNode<T>( current->mData ); // sets mNext = mPrevious = 0
        if( tmp == 0 )
            return *this;

        tmp->mPrevious = mBack;
        if( mBack == 0 )
            mFront = tmp;
        else
            mBack->mNext = tmp;
        mBack = tmp;
    }

    return *this;
}


template <class T>
T& ULList<T>::operator[]( uint32 n ) const
{
    ULListNode<T> *current = mFront;
    for( uint32 k=0; current != 0  &&  k < n; k++ )
        current = current->mNext;

    UL_ASSERT( current != 0 );
    return current->mData;
}


template <class T>
uint32 ULList<T>::size() const
{
    uint32 n = 0;
    for( ULListNode<T> *current=mFront; current != 0; current=current->mNext )
        n++;

    return n;
}


template <class T>
ULListIterator<T> ULList<T>::find( const T& data )
{
    ULListIterator<T> it = begin();
    while( !it.isAtEnd() )
    {
        if( data == *it )
            break;

        ++it;
    }

    return it;
}


template <class T>
void ULList<T>::erase( uint32 n )
{
    ULListNode<T> *current = mFront;
    for( uint32 k=0; current != 0  &&  k < n; k++ )
        current = current->mNext;

    UL_ASSERT( current != 0 );

    ULListIterator<T> it;
    it.mList = this;
    it.mCurrent = current;
    erase( it );
}


template <class T>
void ULList<T>::erase( ULListIterator<T>& it )
{
    if( !it.isAtEnd() )
    {
        ULListNode<T> *node = removeNode( it );
        delete node;
    }
}


template <class T>
void ULList<T>::erase( ULListIterator<T>& start, ULListIterator<T>& stop )
{
    if( start.isAtBeginning() )
    {
        ULListIterator<T> it;
        while( !empty() )
        {
            it = begin();
            if( it.mCurrent == stop.mCurrent )
                break;
            
            erase( it );
        }
    }
    else
    {
        ULListIterator<T> previous = start;
        --previous;

        ULListIterator<T> it = start;
        while( !it.isAtEnd()  &&  it.mCurrent != stop.mCurrent )
        {
            ULListNode<T> *node = removeNode( it );
            delete node;
            it = previous;
            ++it;
        }
    }
}


template <class T>
void ULList<T>::push_front( const T& data )
{
    ULListNode<T> *newNode = new ULListNode<T>;
    if( newNode == 0 )
        return;

    newNode->mData = data;
    newNode->mNext = mFront;
    newNode->mPrevious = 0;
    if( mFront == 0 )
        mBack = newNode;
    else
        mFront->mPrevious = newNode;
    mFront = newNode;
}


template <class T>
void ULList<T>::push_back( const T& data )
{
    ULListNode<T> *newNode = new ULListNode<T>;
    if( newNode == 0 )
        return;

    newNode->mData = data;
    newNode->mNext = 0;
    newNode->mPrevious = mBack;

    if( mBack == 0 )
        mFront = newNode;
    else
        mBack->mNext = newNode;

    mBack = newNode;
}


template <class T>
void ULList<T>::add( const T& data )
{
    ULListIterator<T> it = begin();
    while( !it.isAtEnd() )
    {
        if( data == *it )
            break;

        ++it;
    }

    if( it.isAtEnd() )
        push_back( data );
}


template <class T>
void ULList<T>::insert( ULListIterator<T>& it, const T& data )
{
    ULListNode<T> *node = new ULListNode<T>( data );
    if( node != 0 )
        insertNode( it, node );
}


// Inserts in sorted order.  Does not insert duplicate data.
template <class T>
void ULList<T>::insert( const T& data )
{
    ULListIterator<T> it = begin();
    while( !it.isAtEnd()  &&  *it < data )
        ++it;

    if( it.isAtEnd()  ||  !(*it == data) )
        insert( it, data );
}


template <class T>
void ULList<T>::pop()
{
    if( !empty() )
    {
        ULListIterator<T> it = end();
        --it;
        erase( it );
    }
}


template <class T>
T& ULList<T>::top()
{
    UL_ASSERT( !empty() );
    return mBack->mData;
}


template <class T>
ULListIterator<T> ULList<T>::begin()
{
    ULListIterator<T> it;

    it.mList = this;
    it.mCurrent = mFront;

    return it;
}


template <class T>
ULListConstIterator<T> ULList<T>::cbegin() const
{
    ULListConstIterator<T> it;

    it.mList = this;
    it.mCurrent = mFront;

    return it;
}


template <class T>
ULListIterator<T> ULList<T>::end()
{
    ULListIterator<T> it;

    it.mList = this;
    it.mCurrent = 0;

    return it;
}


template <class T>
ULListConstIterator<T> ULList<T>::cend() const
{
    ULListConstIterator<T> it;

    it.mList = this;
    it.mCurrent = 0;

    return it;
}


template <class T>
ULListIterator<T> ULList<T>::back()
{
    ULListIterator<T> it;

    it.mList = this;
    it.mCurrent = mBack;

    return it;
}


template <class T>
ULListConstIterator<T> ULList<T>::cback() const
{
    ULListConstIterator<T> it;

    it.mList = this;
    it.mCurrent = mBack;

    return it;
}


template <class T>
ULListNode<T> *ULList<T>::removeNode( ULListIterator<T>& it )
{
    ULListNode<T> *node = it.mCurrent;

    UL_ASSERT( it.mList == this );
    if( it.mCurrent != 0  &&  it.mList == this )
    {
        if( it.mCurrent == mFront )
            mFront = mFront->mNext;

        if( it.mCurrent == mBack )
            mBack = mBack->mPrevious;

        if( it.mCurrent->mNext != 0 )
            it.mCurrent->mNext->mPrevious = it.mCurrent->mPrevious;

        if( it.mCurrent->mPrevious != 0 )
            it.mCurrent->mPrevious->mNext = it.mCurrent->mNext;

        if( it.mCurrent->mNext != 0 )
            it.mCurrent = it.mCurrent->mNext;
        else
            it.mCurrent = it.mCurrent->mPrevious;
    }

    return node;
}


template <class T>
void ULList<T>::insertNode( ULListIterator<T>& it, ULListNode<T> *node )
{
    UL_ASSERT( node != 0 );
    UL_ASSERT( it.mList == this );
    if( node == 0  ||  it.mList != this )
        return;

    node->mNext = it.mCurrent;
    if( it.mCurrent == 0 )
    {
        node->mPrevious = mBack;
        if( mBack != 0 )
            mBack->mNext = node;
        mBack = node;
    }
    else
    {
        node->mPrevious = it.mCurrent->mPrevious;
        if( it.mCurrent->mPrevious !=  0 )
            it.mCurrent->mPrevious->mNext = node;

        it.mCurrent->mPrevious = node;
    }

    if( it.mCurrent == mFront )
        mFront = node;

    it.mCurrent = node;
}


template <class T>
ULListNode<T> *ULList<T>::removeFrontNode()
{
    ULListIterator<T> it = begin();
    return removeNode( it );
}

template <class T>
void ULList<T>::sort( int (*func)( T&, T&, void * ), void *context )
{
    // this function implements a qsort algorithm with func() as the
    // comparison function
    
    // it relies on the internal structure of ULList, and tries to perform
    // an efficient sort. No objects are copied, just nodes rearranged and some
    // temporary ULList instances created on the stack
    
    if( mFront == mBack ) // this is a fast way to see if size() is 0 or 1
        return; // a list of 0 or 1 elements is always sorted
    
    // take the first node in the list as the pivot
    ULListNode<T> *pivot = removeFrontNode();
    
    ULList<T> lteqList, gtList; // our two sublists
    
    // iterate the list, removing elements and adding them to the sublists
    while( !empty() )
    {
        ULListNode<T> *cur = removeFrontNode();
        ULList<T>& list =
            func( cur->mData, pivot->mData, context ) <= 0
            ?   lteqList
            :   gtList;
        
        ULListIterator<T> it = list.begin();
        list.insertNode( it, cur );
    }
    
    // the list is divided, now recurse!
    lteqList.sort( func, context );
    gtList.sort( func, context );
    
    // now we rebuild our own list from the pivot and the two others
    
    // add the <= list
    while( !lteqList.empty() )
    {
        ULListNode<T> *node = lteqList.removeFrontNode();
        ULListIterator<T> it = end();
        insertNode( it, node );
    }
    
    // add the pivot
    ULListIterator<T> it = end();
    insertNode( it, pivot );
    
    // add the > list
    while( !gtList.empty() )
    {
        ULListNode<T> *node = gtList.removeFrontNode();
        ULListIterator<T> it = end();
        insertNode( it, node );
    }
    
    // the list is now sorted, and there's no cleanup
}


template <class T>
void ULList<T>::mergeSort(int (*func)(T&, T&, void*), void *context)
{
    // this function implements an in-place mergesort using
    // func() as the comparison function

    // it relies on the internal structure of ULList and
    // avoids copying any objects

    if( mFront == mBack )
        return; // a list of 0 or 1 elements is always sorted

    // first, split ourselves into two sublists
    // we go even/odd so we don't have to iterate twice
    ULList<T> evenList, oddList;
    for( bool odd = false; !empty(); odd = !odd )
    {
        ULListNode<T> *front = removeFrontNode();
        ULList<T>& list = odd ? oddList : evenList;
        ULListIterator<T> it = list.begin();
        list.insertNode( it, front);
    }

    // recurse on the sublists
    evenList.mergeSort( func, context );
    oddList.mergeSort( func, context );

    // now merge
    while( !evenList.empty() && !oddList.empty() )
    {
        ULList<T>& list =
            func( evenList.mFront->mData, oddList.mFront->mData, context) <= 0
            ?   evenList
            :   oddList;
        ULListIterator<T> it = end();
        insertNode( it, list.removeFrontNode() );
    }

    // scoop up stragglers from whichever list has stuff left
    while( !evenList.empty() )
    {
        ULListIterator<T> it = end();
        insertNode( it, evenList.removeFrontNode() );
    }
    while( !oddList.empty() )
    {
        ULListIterator<T> it = end();
        insertNode( it, oddList.removeFrontNode() );
    }

    // all done!
}


// Precondition: The list has been sorted.
template <class T>
void ULList<T>::removeDuplicates()
{
    ULListIterator<T> previous = end();
    ULListIterator<T> current = begin();
    while( !current.isAtEnd() )
    {
        if( !current.isAtEnd()  &&  !previous.isAtEnd()  &&  *current == *previous )
            delete removeNode( current );
        else
            ++current;
    }
}


////////////////////////////////////////////////////////////
// ULHashTableIterator
////////////////////////////////////////////////////////////

template <class K, class V>
class ULHashTable;

template <class K, class V>
class UL_API ULHashTableIterator
{
    friend class ULHashTable<K,V>;

private:
    const ULHashTable<K,V>              *mHashTable;
    uint32                              mHashIndex;
    ULListConstIterator< ULPair<K,V> >  mListIterator;

public:
                                        ULHashTableIterator();
                                        ULHashTableIterator( const ULHashTableIterator& it );
                                        ~ULHashTableIterator();

    const ULHashTableIterator&          operator=( const ULHashTableIterator& it );
    const ULPair<K,V>&                  operator*() const;
    void                                operator++();
    void                                operator++( int );

    bool                                isAtEnd() const { return (mHashTable == 0  ||  mHashIndex >= mHashTable->mTableSize); }

    uint32                              getHashIndex() const { return mHashIndex; }
};


template <class K, class V>
ULHashTableIterator<K,V>::ULHashTableIterator()
    : mHashTable(0), mHashIndex(0)
{
}


template <class K, class V>
ULHashTableIterator<K,V>::ULHashTableIterator( const ULHashTableIterator& it )
    : mHashTable(0), mHashIndex(0)
{
    *this = it;
}


template <class K, class V>
ULHashTableIterator<K,V>::~ULHashTableIterator()
{
}


template <class K, class V>
const ULHashTableIterator<K,V>& ULHashTableIterator<K,V>::operator=( const ULHashTableIterator& it )
{
    if( &it != this )
    {
        mHashTable = it.mHashTable;
        mHashIndex = it.mHashIndex;
        mListIterator = it.mListIterator;
    }

    return *this;
}


template <class K, class V>
const ULPair<K,V>& ULHashTableIterator<K,V>::operator*() const
{
    UL_ASSERT( !isAtEnd() );
    return *mListIterator;
}


template <class K, class V>
void ULHashTableIterator<K,V>::operator++()
{
    if( mHashTable == 0 )
        return;

    ++mListIterator;
    if( mListIterator.isAtEnd() )
    {
        mHashIndex++;
        mListIterator = mHashTable->mTable[mHashIndex].cbegin();
        while( mHashIndex < mHashTable->mTableSize  &&  mListIterator.isAtEnd() )
        {
            mHashIndex++;
            mListIterator = mHashTable->mTable[mHashIndex].cbegin();
        }    
    }
}


template <class K, class V>
void ULHashTableIterator<K,V>::operator++( int )
{
    ++(*this);
}


////////////////////////////////////////////////////////////
// ULHashTable
////////////////////////////////////////////////////////////

template <class K, class V>
class UL_API ULHashTable
{
    friend class ULHashTableIterator<K,V>;

private:
    ULList< ULPair<K,V> >   *mTable;
    uint32                  mTableSize;

public:
                    ULHashTable();
                    ULHashTable( uint32 tableSize );
                    ~ULHashTable();

    void            clear();
    ULHashTable<K,V>& operator=( const ULHashTable<K,V>& table );

    uint32          getTableSize() const;
    bool            resize( uint32 tableSize );

    void            add( const K& key, const V& value );
    void            add( const ULPair<K,V>& keyValuePair );

    bool            find( const K& key, V& value ) const;
    bool            find( const K& key ) const;

    ULHashTableIterator<K,V>    end() const;
    ULHashTableIterator<K,V>    begin() const;

    void            getStats( double& avgListLength, uint32& maxListLength );
};


template <class K, class V>
ULHashTable<K,V>::ULHashTable()
    : mTable(0), mTableSize(0)
{
}


template <class K, class V>
ULHashTable<K,V>::ULHashTable( uint32 tableSize )
    : mTable(0), mTableSize(0)
{
    if( tableSize > 0 )
    {
        mTable = new ULList< ULPair<K,V> >[tableSize];
        if( mTable != 0 )
            mTableSize = tableSize;
    }
}


template <class K, class V>
ULHashTable<K,V>::~ULHashTable()
{
    delete [] mTable;
}


/**
 * Removes all items from the hash table, leaving the table the same size
 * as before, but empty.
 */

template <class K, class V>
void ULHashTable<K,V>::clear()
{
    if( mTable != 0 )
    {
        for( uint32 k=0; k < mTableSize; k++ )
            mTable[k].clear();
    }
}


template <class K, class V>
ULHashTable<K,V>& ULHashTable<K,V>::operator=( const ULHashTable<K,V>& table )
{
    delete [] mTable;
    mTable = 0;
    mTableSize = 0;
    if( table.mTableSize > 0 )
        mTable = new ULList< ULPair<K,V> >[table.mTableSize];

    if( mTable != 0 )
    {
        mTableSize = table.mTableSize;
        for( uint32 k=0; k < mTableSize; k++ )
        {
            mTable[k] = table.mTable[k];
        }
    }

    return *this;
}


template <class K, class V>
uint32 ULHashTable<K,V>::getTableSize() const
{
    return mTableSize;
}


template <class K, class V>
bool ULHashTable<K,V>::resize( uint32 tableSize )
{
    if( tableSize == 0  ||  tableSize == mTableSize )
        return true;

    ULList< ULPair<K,V> > *newTable = new ULList< ULPair<K,V> >[tableSize];
    if( newTable == 0 )
        return false;

    if( mTableSize != 0 )
    {
        uint32 hashValue;
        ULListConstIterator< ULPair<K,V> > it;
        for( uint32 k=0; k < mTableSize; k++ )
        {
            it = mTable[k].cbegin();
            while( it != mTable[k].cend() )
            {
                hashValue = (*it).mFirst.hash( tableSize );
                UL_ASSERT( hashValue < tableSize );
                newTable[hashValue].push_back( *it );
                ++it;
            }
        }
    }

    delete [] mTable;
    mTable = newTable;
    mTableSize = tableSize;

    return true;
}


template <class K, class V>
void ULHashTable<K,V>::add( const K& key, const V& value )
{
    ULPair<K,V> keyValuePair;
    keyValuePair.mFirst = key;
    keyValuePair.mSecond = value;
    add( keyValuePair );
}


// If the key isn't present, add a new item to the table.
// If the key is present, change its value.
template <class K, class V>
void ULHashTable<K,V>::add( const ULPair<K,V>& keyValuePair )
{
    uint32 hashValue = keyValuePair.mFirst.hash( mTableSize );
    UL_ASSERT( hashValue < mTableSize );
    if( hashValue < mTableSize )
    {
        ULListIterator< ULPair<K,V> > it = mTable[hashValue].begin();
        while( it != mTable[hashValue].end() )
        {
            if( keyValuePair.mFirst == (*it).mFirst )
            {
                (*it).mSecond = keyValuePair.mSecond;
                return;
            }

            ++it;
        }

        mTable[hashValue].push_back( keyValuePair );
    }
}


template <class K, class V>
bool ULHashTable<K,V>::find( const K& key, V& value ) const
{
    uint32 hashValue = key.hash( mTableSize );
    UL_ASSERT( hashValue < mTableSize );
    if( hashValue < mTableSize )
    {
        ULListConstIterator< ULPair<K,V> > it = mTable[hashValue].cbegin();
        while( it != mTable[hashValue].cend() )
        {
            if( key == (*it).mFirst )
            {
                value = (*it).mSecond;
                return true;
            }

            ++it;
        }
    }

    return false;
}


template <class K, class V>
bool ULHashTable<K,V>::find( const K& key ) const
{
    uint32 hashValue = key.hash( mTableSize );
    UL_ASSERT( hashValue < mTableSize );
    ULListConstIterator< ULPair<K,V> > it = mTable[hashValue].cbegin();
    while( it != mTable[hashValue].cend() )
    {
        if( key == (*it).mFirst )
            return true;

        ++it;
    }

    return false;
}

template <class K, class V>
ULHashTableIterator<K,V> ULHashTable<K,V>::end() const
{
    ULHashTableIterator<K,V> it;
    it.mHashTable = this;
    it.mHashIndex = mTableSize;

    return it;
}


template <class K, class V>
ULHashTableIterator<K,V> ULHashTable<K,V>::begin() const
{
    ULHashTableIterator<K,V> it;
    it.mHashTable = this;
    it.mHashIndex = 0;
    it.mListIterator = mTable[it.mHashIndex].cbegin();
    while( it.mHashIndex < mTableSize  &&  it.mListIterator.isAtEnd() )
    {
        it.mHashIndex++;
        it.mListIterator = mTable[it.mHashIndex].cbegin();
    }

    return it;
}


template <class K, class V>
void ULHashTable<K,V>::getStats( double& avgListLength, uint32& maxListLength )
{
    maxListLength = 0;
    uint32 listLengthTotal = 0;
    uint32 nNonemptyLists = 0;
    for( uint32 k=0; k < mTableSize; k++ )
    {
        uint32 listLength = mTable[k].size();
        listLengthTotal += listLength;
        if( listLength > maxListLength )
            maxListLength = listLength;
        if( listLength > 0 )
            nNonemptyLists++;
    }

    if( nNonemptyLists > 0 )
        avgListLength = double(listLengthTotal) / double(nNonemptyLists);
    else
        avgListLength = 0;
}


////////////////////////////////////////////////////////////
// ULVector
////////////////////////////////////////////////////////////

template <class T>
class UL_API ULVector
{
private:
    T               **mVector;
    uint32          mVectorSize;

public:
                    ULVector();
                    ULVector( uint32 vectorSize );
                    ULVector( const ULVector<T>& vector );
                    ~ULVector();

    ULVector<T>&    operator=( const ULVector<T>& vector );
    void            clear();
    T&              operator[]( uint32 n ) const;
    uint32          size() const;
    void            resize( uint32 newSize );
};


template <class T>
ULVector<T>::ULVector()
{
    mVector = 0;
    mVectorSize = 0;
}


template <class T>
ULVector<T>::ULVector( uint32 vectorSize )
{
    mVector = 0;
    mVectorSize = 0;
    resize( vectorSize );
}


template <class T>
ULVector<T>::ULVector( const ULVector<T>& vector )
{
    mVector = 0;
    mVectorSize = 0;
    *this = vector;
}


template <class T>
ULVector<T>::~ULVector()
{
    clear();
}


template <class T>
void ULVector<T>::clear()
{
    if( mVector != 0 )
    {
        for( uint32 k=0; k < mVectorSize; k++ )
            delete mVector[k];
        delete [] mVector;
    }

    mVector = 0;
    mVectorSize = 0;
}


template <class T>
ULVector<T>& ULVector<T>::operator=( const ULVector<T>& vector )
{
    if( this == &vector )
        return *this;

    clear();
    resize( vector.size() );
    if( mVector != 0 )
    {
        for( uint32 k=0; k < mVectorSize; k++ )
            (*this)[k] = vector[k];
    }

    return *this;
}


template <class T>
T& ULVector<T>::operator[]( uint32 n ) const
{
    UL_ASSERT( mVector != 0 );
    UL_ASSERT( n < mVectorSize );

    if( mVector[n] == 0 )
    {
        mVector[n] = new T;
    }

    return *mVector[n];
}


template <class T>
uint32 ULVector<T>::size() const
{
    return mVectorSize;
}


template <class T>
void ULVector<T>::resize( uint32 newSize )
{
    if( newSize != mVectorSize )
    {
        if( newSize == 0 )
            clear();
        else
        {
            typedef T *TPtr;
            T **newVector = new TPtr[newSize];
            if( newVector != 0 )
            {
                uint32 k;
                for( k=0; k < newSize  &&  k < mVectorSize; k++ )
                {
                    newVector[k] = mVector[k];
                    mVector[k] = 0;
                }

                for( ; k < newSize; k++ )
                    newVector[k] = 0;

                clear();
                mVector = newVector;
                mVectorSize = newSize;
            }
        }
    }
}

#endif
