/**
 * @file ulsegment.h
 * @brief The interface for the <code>ULSegment</code> class.
 * @author Jeff Ondich
 */

#ifndef ULSEGMENT_H
#define ULSEGMENT_H

/**
 * @class ULSegment
 * @brief Each object of this class represents a single
 * segment from a segmented file (which is in turn represented
 * by a <code>ULSegmentedFile</code> object).
 */

#include "ulfile.h"
#include "ulcompressor.h"

const uint32 kULDefaultCacheSize = 8;
const uint32 kULDDefaultBlockSize = 0x400;

struct ULCacheEntry
{
    uint32        mBlockNumber;
    uint32      mNBytes;
    uint8        *mUncompressedData;
    
                ULCacheEntry();
                ULCacheEntry( uint32 blockSize );
                ~ULCacheEntry();

    void        clear();
};


class ULSegment
{
    friend class ULSegmentedFile;

private:
    ULFile          *mFile;
    ULCompressor    *mCompressor;
    uint32          mSegmentType;
    uint32          mStart;
    uint32          mUncompressedSize;
    uint32          mCompressedSize;
    uint32          mBlockSize;
    bool            mIsCompressed;
    uint32          mNBlocks;

    // Used by all memory management schemes.
    uint8           *mTmpBuffer;

    // For mSegmentType == kULLoadedSegmentType.
    uint8           *mBlockValid;
    uint8           *mSegmentBuffer;

    // For mSegmentType == kULCachedSegmentType.
    ULCacheEntry    **mCache;
    uint32          mCacheSize;

    // For writing.
    enum { kULDefaultSegmentSize = 128 };
    uint32          mCapacity;

    void            clear();

    void            initMemory();
    bool            isBlockReady( uint32 blockNumber );
    bool            loadBlock( uint32 blockNumber );
    uint8           getByteFromBlock( uint32 blockNumber, uint32 blockOffset );

public:
    enum { kULDoNotOpenSegmentType=0, kULLoadedSegmentType, kULCachedSegmentType };

                    ULSegment();
                    ULSegment( ULFile *file,
                               ULCompressor *compressor,
                               uint32 start,
                               uint32 uncompressedSize, 
                               uint32 compressedSize,
                               uint32 blockSize,
                               uint32 type=kULLoadedSegmentType,
                               uint32 cacheSize=kULDefaultCacheSize );

                    ~ULSegment();

    void            init( ULFile *file,
                          ULCompressor *compressor,
                          uint32 start,
                          uint32 uncompressedSize, 
                          uint32 compressedSize,
                          uint32 blockSize,
                          uint32 type=kULLoadedSegmentType,
                          uint32 cacheSize=kULDefaultCacheSize );

    uint32          getStart();
    uint32          getUncompressedSize();
    uint32          getCompressedSize();
    uint32          getBlockSize();
    bool            isCompressed();

    uint8           getByte( uint32 offset );

    void            putByte( uint8 byte, uint32 offset );
    void            flush();
};

#endif

