/**
 * @file uldatamanager.h
 * @brief The interface for the ULDataManager class.
 * @author Jeff Ondich
 */

#ifndef ULDATAMANAGER_H
#define ULDATAMANAGER_H

#include "ulstems.h"
#include "ulmodules.h"
#include "ulterm.h"
#include "ulstringpool.h"
#include "ulrendereroptions.h"
#include "ulnumbertranslator.h"

class ULStemmer;
class ULConjugator;
struct ULConjugationResult;
class ULDManager;
class ULDMIterator;
class ULCompressor;
class ULTokenizer;
class ULRegistrar;
struct ULRegistrationInfo;
class ULD;
struct ULDMRegexOptions;

enum
{
    kULAddDictionaryNoError=0,
    kULAddDictionaryWrongProduct,
    kULAddDictionaryWrongLanguages,
    kULAddDictionaryULD40,
    kULAddDictionaryULD50,
    kULAddDictionaryNotULD,
    kULAddDictionaryPastDropDeadDate,
    kULGenericError
};

class ULHeadwordMatch {
public:
    ULHeadwordMatch();
    ULHeadwordMatch( const ULHeadwordMatch& headwordMatch );
    virtual ~ULHeadwordMatch();
    
    virtual void clear();
    virtual ULHeadwordMatch& operator=( const ULHeadwordMatch& h );
    
    const ulstring& getHeadword() const;
    void setHeadword( const ulstring& s );

    bool getStemmed() const;
    void setStemmed( bool b );

    const ulstring& getPOSP() const;
    void setPOSP( const ulstring& s );

private:
    ulstring mHeadword;
    bool mStemmed;
    ulstring mPOSP;
};

/**
 * @class ULDataManager
 * @brief The main class used to access ULAPI services.
 *
 * The ULDataManager class gives you a single
 * interface through which to request almost all the services
 * provided by ULAPI.  Most applications of ULAPI will use
 * a single object of type ULDataManager, adding
 * dictionary and conjugator modules at startup and using the
 * ULDataManager services as appropriate.
 *
 * ULDataManager offers several collections of
 * services:
 * <ul>
 *    <li>Definition/translation.  For each .uld file
 *    installed on the user's system, call addDictionaryModule().
 *    After that, you can use begin(), end(),
 *    find(), initRegex(), and addTerm(),
 *    to work with the dictionary data.</li>
 *    <li>Conjugation/stemming.  For each .ulc file installed on
 *    the user's system, call addStemmingModule().
 *    Then, find() will be able to do stemmed searches,
 *    and you'll also be able to use getMatchingVerbs(),
 *    getConjugation(), etc.</li>
 *    <li>Registration key management (not supported for most
 *    versions of ULAPI).  Registration keys and days-used information
 *    for various Ultralingua data modules are stored in encrypted
 *    strings called <em>key storage strings</em>.  At startup of a
 *    ULAPI-based application, you should collect all the key storage
 *    strings currently stored on the user's system, and add them
 *    to the ULDataManager object via addKeyStorageString().
 *    At startup, you should also call setTrialPeriod()
 *    to specify the number of days the user is allowed to use
 *    products before registering, and setPlatform() to
 *    specify the platform on which the application is running
 *    (<code>kULWindows</code>, <code>kULMacintosh</code>, etc.).
 *    Then call useProduct() whenever the user starts
 *    searching in a given dictionary, and submitRegistrationKey()
 *    whenever a user tries to register a product.  Before the application
 *    shuts down, call getKeyStorageStrings() and store
 *    the resulting strings wherever you are hiding registration
 *    information on the user's computer.</li>
 * </ul>
 *
 * Data modules on Palm OS devices are stored in Palm Database (.pdb) format.
 * When the documentation for ULDataManager refers to .ulc or .uld
 * files, keep in mind that the corresponding data files for Palm are .pdb files,
 * usually named in such a way as to clarify whether they are dictionary modules
 * or conjugation/stemming modules. 
 */

class UL_API ULDataManager
{
private:
    ULList< ULPair<ULModuleIdentifier,ULLanguage> >     mStemmingModuleList;
    ULList< ULPair<ULLanguage,ULLanguage> >             mAllowedLanguagePairs;
    ULList<ulstring> mLocalizedStrings;
    ULList<uint32>   mAllowedProducts;
    ULStringPool     mRendererStrings;

    ULStemmer        *mStemmer;
    ULConjugator     *mConjugator;
    ULDManager       *mULDManager;
    ULCompressor     *mCompressor;
    ULTokenizer      *mTokenizer;
    ULRegistrar      *mRegistrar;

    uint32           mPlatform;

    bool             openConjugator( ULLanguage lang );
    ULD              *getAdditionsULD( ULLanguage lang1, ULLanguage lang2, uint32 product, const ulstring& dataDirectory );
    void             deleteDictionaryModules();
    void             closeDictionary( ULLanguage lang1, ULLanguage lang2, uint32 product );
    bool             mergeable( const ULModuleType& module1, const ULModuleType& module2 );

public:
    static ULDataManager *create();

                     ULDataManager();
                     ~ULDataManager();

    bool             init();

    void             clear();
    void             closeAllDictionaries();

    // Handy once in a while
    ULCompressor*    getCompressor() { return mCompressor; }
    ULTokenizer*     getTokenizer() { return mTokenizer; }

    // Detecting whether languages and language pair dictionaries are installed.
    bool             isLanguageAvailable( ULLanguage lang, bool& expired );
    bool             isLanguagePairAvailable( ULLanguage lang1, ULLanguage lang2, bool& expired );

    // Getting information about a module from its moduleID.
    bool             getDictionaryModuleInfo( const ULModuleIdentifier& moduleID, ULLanguage& red, ULLanguage& green, uint32& version );
    bool             getDictionaryModuleInfo( const ULModuleIdentifier& path, ULLanguage& red, ULLanguage& green, uint32& product, uint32& version );
    bool             getStemmingModuleInfo( const ULModuleIdentifier& moduleID, ULLanguage& lang, uint32& version );

    // Setting up and tearing down.
    int              addDictionaryModule( const ULModuleIdentifier& moduleID );
    bool             addStemmingModule( const ULModuleIdentifier& moduleID );

    // If there are no "allowed language pairs", then any language pair can be added to
    // the key storage string list and the dictionary list.  But if there are allowed
    // language pairs, then those are the *only* pairs allowed.  This enables us, for
    // example, to restrict a special version of the program to using only French-Spanish.
    void             addAllowedLanguagePair( ULLanguage lang1, ULLanguage lang2 );

    // If there are no "allowed products", then only kULDefaultProduct products will be
    // allowed to be added via addDictionaryModule.  Otherwise, only modules that match
    // something in the mAllowedProducts list will be allowed.
    void             addAllowedProduct( uint32 product );

    // In Palm, it takes a while to call ULD::attachToDatabase.  But since we want to
    // be able to quickly identify the available databases, their languages, and
    // their registration status at startup, we need to separate the "add the module to
    // the list" operation from the "open up the database and start loading its contents"
    // operation.  Thus, addDictionaryModule does not call ULD::attachToDatabase--that's
    // reserved for attachAllModules.
    void             attachAllModules();

    // Getting lists (with subtle variations) of the installed dictionary modules.
    void             getDictionaryList( ULList<ULModuleType>& moduleList );
    void             getProductList( ULList<ULModuleType>& moduleList );
    void             getModuleList( ULList<ULModuleType>& moduleList );

    // For debugging and performance tuning with cache-based ULDs.
    void             reportHitStats();

    // Setting up registration info.
    void             setPlatform( uint32 platform ) { mPlatform = platform; }
    void             setTrialPeriod( uint32 trial );
    void             setTrialType( int trialType );
    bool             addKeyStorageString( const ulstring& keyStorageString );

    // Registration information access.
    uint32           getPlatform() { return mPlatform; }
    void             getRegistrationKeys( ULList<ulstring>& keyList );
    void             getKeyStorageStrings( ULList<ulstring>& kssList );
    bool             isExpired( ULLanguage lang1, ULLanguage lang2, uint32 product ) const;
    bool             isRegistered( ULLanguage lang1, ULLanguage lang2, uint32 product ) const;
    uint32           daysUsed( ULLanguage lang1, ULLanguage lang2, uint32 product ) const;
    uint32           daysLeft( ULLanguage lang1, ULLanguage lang2, uint32 product ) const;
    ulstring         userName( ULLanguage lang1, ULLanguage lang2, uint32 product ) const;
    void             useProduct( ULLanguage lang1, ULLanguage lang2, uint32 product, uint32 year, uint32 month, uint32 day );
    bool             submitRegistrationKey( const ulstring& key, uint32 year, uint32 month, uint32 day, ULRegistrationInfo& regInfo );
    void             removeRegistrationKey( const ulstring& key );

    // ULStemmer access functions.
    void             getStems( ULList< ULPair<ULStem,ULDMIterator> >& stemList,
                               const ulstring& searchString,
                               ULLanguage stemmingLanguage,
                               ULLanguage otherLanguage,
                               uint32 product,
                               int verbMatchingStyle=kULCFirstMatchPerVerb );

    // ULConjugator access functions.
    bool             isConjugationLanguageAvailable( ULLanguage lang );
    void             getConjugationLanguageList( ULList<ULLanguage>& languageList );

    void             getConjugationModuleList( ULList<ULModuleType>& moduleList );

    bool             getVerb( ULLanguage lang,
                              const ulstring& searchString,
                              ULVerb& verb );

    void             getVerbs( ULLanguage lang,
                               const ulstring& searchString,
                               ULList<ULVerb>& verbList );

    void             getMatchingVerbs( ULLanguage lang,
                                       const ulstring& searchString,
                                       ULList<ulstring>& verbs,
                                       bool clearList=false,
                                       uint32 maxVerbCount=0,
                                       bool eliminateDuplicates=false,
                                       bool includeStemmedResults=false );

    void             getAllVerbs( ULLanguage lang, ULList<ulstring>& verbs );

    bool             getConjugation( ULLanguage lang,
                                     const ULVerb& verb,
                                     ULTense tense,
                                     ULPerson person,
                                     ulstring& conjugatedVerb );

    bool             getConjugation( ULLanguage lang,
                                     const ULVerb& verb,
                                     ULTense tense,
                                     ULPerson person,
                                     ULList<ulstring>& conjugationList );

    bool             getFullConjugation( ULLanguage lang,
                                         const ulstring& infinitive,
                                         uint32 classID,
                                         ULList<ULConjugationResult>& conjugationList );

    void             getVerbDetails( ULLanguage lang, const ULVerb& verb, ulstring& details ); 

    void             getTenses( ULLanguage lang, const ULVerb& v, ULList<ULTense>& tenseList, bool includeParticiples=false );
    void             getAllTenses( ULLanguage lang, ULList<ULTense>& tenseList, bool includeParticiples=false );
    void             getTenseName( ULLanguage lang, ULTense tense, ulstring& name );
    void             getPersons( ULLanguage lang, const ULVerb& v, ULTense tense, ULList<ULPerson>& personList );
    void             getPersonName( ULLanguage lang, ULPerson person, ulstring& name );
    void             getEnglishPersonDescription( ULLanguage lang, ULPerson person, ulstring& description );
    void             getConjugationDisplayText( ULLanguage lang, const ULVerb& verb, ulchar separator, ULTense tense, ULPerson person, ulstring& text );
    void             getKlingonMorphology( ulstring& searchString, uint32 partOfSpeechRank, bool ignorePrefix, ULList< ULPair<ulstring, ulstring> >& morphemes );


    // ULNumberTranslator access functions.
    bool             isNumberTranslatorLanguageAvailable( ULLanguage lang );
    void             getTranslatedNumbers( ULList< ULPair<ulstring, ulstring> >& results,
                                           const ulstring& query,
                                           ULLanguage lang,
                                           uint32 maxResults=ULNumberTranslator::kULMaxNumberTranslations );

    // ULDManager access functions.
    ULDManager       *getULDManager() { return mULDManager; }

    uint32           getNModules();

    void             begin( ULLanguage searchLanguage,
                            ULLanguage translationLanguage,
                            uint32 product,
                            ULDMIterator& it );

    void             end( ULLanguage searchLanguage,
                          ULLanguage translationLanguage,
                          uint32 product,
                          ULDMIterator& it );

    void             find( const ulstring& searchString,
                           ULLanguage searchLanguage,
                           ULLanguage translationLanguage,
                           uint32 product,
                           ULDMIterator& it );

    void             find( const ULTerm& searchTerm, uint32 product, ULDMIterator& it );

    void             getApproximateMatches( const ulstring& searchString,
                                            ULLanguage searchLanguage,
                                            ULLanguage translationLanguage,
                                            uint32 product,
                                            uint32 requiredPrefixLength,
                                            ULList<ulstring>& headwords );

    void             getApproximateMatches( const ulstring& searchString,
                                            ULLanguage searchLanguage,
                                            ULLanguage translationLanguage,
                                            uint32 product,
                                            uint32 requiredPrefixLength,
					    bool eliminateExactMatches,
					    uint32 stringComparisonOptions,
                                            ULList<ulstring>& headwords );


    void             getAdditionsPartsOfSpeech( ULList< ULPair<ulstring,uint32> >& pospList,
                                                ULLanguage lang1,
                                                ULLanguage lang2,
                                                uint32 product );

    // ULD rendering/XHTML-generation support.
    enum {
        kXHTMLShowExactMatchesOnly = 1,
        kXHTMLRegexDisplay = 1 << 1
    };
    
    enum { kXHTMLStringEmpty = 0,
           kXHTMLStringPhrases,
           kXHTMLStringSynonyms,
           kXHTMLStringHomonyms,
           kXHTMLStringUsageNote,
           kXHTMLStringEtymology,
           kXHTMLStringAlternateSpelling,
           kXHTMLStringAntonyms,
           kXHTMLStringVerbConjugation,
           kNXHTMLStrings };

    void             clearRendererStrings();
    void             setRendererString( int stringId, const ulstring& s );

    void             getMatchingHeadwords( const ulstring& searchString,
                                           ULLanguage searchLanguage,
                                           ULLanguage translationLanguage,
                                           uint32 product,
                                           ULRendererOptions& options,
                                           /* Out: */
                                           ULList<ULHeadwordMatch> &headwordList );
    
    void             getDictionarySearchResults( const ulstring& searchString,
												ULLanguage searchLanguage,
												ULLanguage translationLanguage,
												uint32 product,
												ULRendererOptions& options,
												/* Out: */
												ULList<ulstring>& approximateMatches,
												ULList< ULPair<ULStem,ULDMIterator> >& stemList,
												ulstring& stemmingXHTML,
												ulstring& browsingXHTML );

	void			 getIPadXHTML( const ulstring& searchString,
								  ULLanguage searchLanguage,
								  ULLanguage translationLanguage,
								  uint32 product,
								  uint32 termsToRender,
								  /* Out: */
								  ulstring &browsingXHTML );

	void			 getIPadXHTML( ULDMIterator& uldmIt,
								  uint32 termsToRender,
								  /* Out: */
								  ulstring &browsingXHTML );
	
	void			 renderIPadVerbTable ( ULLanguage lang,
                                           const ULVerb& verb,
                                           ulstring& verbTable );
	
    void             getHeadwordSearchResults( const ulstring& searchString,
											  ULLanguage searchLanguage,
											  ULLanguage definitionLanguage,
											  uint32 product,
											  uint32 maxBrowsingResults,
											  uint32 approximateRequiredPrefixLength,
											  /* Out: */
											  ULList<ulstring>& approximateMatches,
											  ULList<ulstring>& stemmingMatches,
											  ULList<ulstring>& browsingMatches );

    void             getIPhoneHeadwordSearchResults( const ulstring& searchString,
													ULLanguage searchLanguage,
													ULLanguage definitionLanguage,
													uint32 product,
													uint32 maxBrowsingResults,
													uint32 approximateRequiredPrefixLength,
													/* Out: */
													ULList<ulstring>& approximateMatches,
													ULList<ulstring>& stemmingMatches,
													ULList<ulstring>& browsingMatches );
	
    void             appendTermXHTML( ULDMIterator& it,
                                      uint32 product,
                                      ULRendererOptions& options,
                                      ULHashTable<ULDMIterator, uint32>& displayedTerms,
                                      /* Out: */
                                      ulstring& xhtml,
                                      uint32& nDisplayedTerms );
    
    void             getGEICODictionaryXHTML( const ulstring& searchString,
                                        ULLanguage searchLanguage,
                                        ULLanguage translationLanguage,
                                        uint32 product,
                                        ULRendererOptions& options,
                                        /* Out: */
                                        ulstring& xhtml,
                                        ULList<ULDMIterator> &iteratorsForDisplayedTerms,
                                        int32 &termIDOfFirstPhraseMatch );
    
    void             appendGEICOTermXHTML( ULDMIterator& it,
                                     const ulstring &searchString,
                                     ULLanguage searchLanguage,
                                     ULLanguage translationLanguage,
                                     uint32 product,
                                     ULRendererOptions& options,
                                     /* Out: */
                                     ulstring& xhtml,
                                     int32 &termIDOfFirstPhraseMatch,
                                     ULList<ULDMIterator> &iteratorsForDisplayedTerms,
                                     ULHashTable<ULDMIterator, uint32>& displayedTerms,
                                     uint32& numDisplayedTerms );
    
    void             getDictionaryXHTML( const ulstring& searchString,
                                        ULLanguage searchLanguage,
                                        ULLanguage translationLanguage,
                                        uint32 product,
                                        ULRendererOptions& options,
                                        /* Out: */
                                        ulstring& xhtml,
                                        ULList<ULDMIterator> &iteratorsForDisplayedTerms,
                                        int32 &termIDOfFirstPhraseMatch );
    
    void             appendTermXHTML( ULDMIterator& it,
                                     const ulstring &searchString,
                                     ULLanguage searchLanguage,
                                     ULLanguage translationLanguage,
                                     uint32 product,
                                     ULRendererOptions& options,
                                     /* Out: */
                                     ulstring& xhtml,
                                     int32 &termIDOfFirstPhraseMatch,
                                     ULList<ULDMIterator> &iteratorsForDisplayedTerms,
                                     ULHashTable<ULDMIterator, uint32>& displayedTerms,
                                     uint32& numDisplayedTerms );
    
#ifdef UL_REGEX
    void             initRegex( const ulstring& searchString,
                                ULLanguage searchLanguage,
                                ULLanguage translationLanguage,
                                uint32 product,
                                const ULDMRegexOptions& options,
                                ULDMIterator& it );
#endif

    bool             addTerm( const ulstring& text,
                              const ulstring& partOfSpeech,
                              uint32 partOfSpeechRank,
                              const ulstring& translationText,
                              const ulstring& translationPartOfSpeech,
                              uint32 translationPartOfSpeechRank,
                              ULLanguage lang,
                              ULLanguage translationLang,
                              uint32 product,
                              const ulstring& dataDirectory );

    bool             addTerms( const ULList<ULTerm>& termList, uint32 product, const ulstring& dataDirectory );
};

#endif
