/**
 * @file ulsegmentedfile.h
 * @brief The interface for the <code>ULSegmentedFile</code> class.
 * @author Jeff Ondich
 */

#ifndef ULSEGMENTEDFILE_H
#define ULSEGMENTEDFILE_H

/**
 * @class ULSegmentedFile
 * @brief This class describes a file that is broken into
 * segments.
 * 
 * A segmented file begins with a segment header, followed
 * by the segments themselves.  The segment header looks
 * like this (sixteen bytes per row, all integers stored
 * in big-endian byte order):
 *
 * <blockquote>
 * 0x05E9F17E | [# segments] | 0x00000000 | 0x00000000<br />
 * [starting offset] | [uncompressed size] | [compressed size] | [block size]<br />
 * (one row per segment)
 * </blockquote>
 *
 * The "starting offset" is the byte offset within the file at which
 * the segment begins (note that this offset includes the segmentation
 * header, so the first segment's starting offset will be 32 for a one-segment
 * file, 48 for a two-segment file, etc.).  Compressed segments are
 * compressed one block at a time.  The left-most bit of the "block size"
 * field determines whether the segment uses compression (1 = compressed,
 * 0 = not compressed).  The rest of the block size field
 * (i.e. (block size) & 0x7FFFFFFF) is the uncompressed size of the blocks.
 *
 * An uncompressed segment is just data, starting at the "starting offset"
 * within the file, and consisting of the "uncompressed size" number of
 * bytes.
 *
 * A compressed segment, on the other hand, consists of a "block map"
 * followed by a sequence of compressed blocks.  Consider a compressed
 * segment before compression.  It has "uncompressed size" bytes, which
 * we divide into blocks of a fixed size ((block size) & 0x7FFFFFFF).
 * We compress these blocks, yielding a sequence of compressed blocks
 * of various sizes.  If we want to be able to extract these blocks,
 * we'll need to know how big they are.  Thus, each compressed segment
 * begins with a block map, which is a sequence of 4-byte big endian
 * integers pointing to the offset within the segment at which each
 * successive block begins.  Suppose, for example, that the first three
 * blocks have compressed sizes of 850, 650, and 750.  Then the block
 * map for this segment will begin with the integers 0, 850, 1500, and
 * 2250 for the first four blocks.
 */

#include "ulfile.h"
#include "ulcompressor.h"
#include "ulsegment.h"

const uint32 kULSegmentedFileMagicNumber = 0x05E9F17E;

struct ULSegmentDescriptor
{
    uint32 mStart;
    uint32 mUncompressedSize;
    uint32 mCompressedSize;
    uint32 mBlockSize;
    bool mUsingCompression;
};

class ULSegment;

class ULSegmentedFile
{
private:
    ULFile          *mFile;
    ULCompressor    *mCompressor;
    ULSegment       **mSegments;
    uint32          mNSegments;

public:
                    ULSegmentedFile();
                    ULSegmentedFile( ULCompressor *compressor );
                    ~ULSegmentedFile();

    void            setCompressor( ULCompressor *compressor );

    bool            open( const ULModuleIdentifier& moduleID,
                          uint32 openMode,
                          uint32 nSegments,
                          const uint32 *segmentTypes,
                          const uint32 *cacheSizes );

    void            close();
    bool            isOpen() const;

    uint8           getByte( uint32 segmentIndex, uint32 offset );
    bool            getULChar( uint32 segmentIndex, uint32& offset, uint32 limit, ulchar& ch );
    uint16          getInt16( uint32 segmentIndex, uint32 offset );
    uint32          getInt24( uint32 segmentIndex, uint32 offset );
    uint32          getInt32( uint32 segmentIndex, uint32 offset );
    uint32          getBase255Int24( uint32 segmentIndex, uint32 offset );
    uint32          getBase255Int32( uint32 segmentIndex, uint32 offset );
    void            getString( ulstring& s,
                               uint32 segmentIndex,
                               uint32& offset,
                               ulchar delimiter=0 );

    uint32          getNSegments() const;
    uint32          getSegmentSize( uint32 segmentIndex );
    uint32          getCompressedSegmentSize( uint32 segmentIndex );
    uint32          getSegmentBlockSize( uint32 segmentIndex );
    bool            isCompressed( uint32 segmentIndex );

    void            putByte( uint32 segmentIndex, uint32& offset, uint8 byte );
    void            putInt16( uint32 segmentIndex, uint32& offset, uint16 n );
    void            putInt24( uint32 segmentIndex, uint32& offset, uint32 n );
    void            putInt32( uint32 segmentIndex, uint32& offset, uint32 n );
    void            putString( uint32 segmentIndex, uint32& offset, const ulstring& s );
    void            putInt32( uint32 n );
    static void     putInt32( ULFile *out, uint32 n );
    void            setCompression( uint32 segmentIndex, bool useCompression );
    uint32          seek( int32 offset, int origin=ULFile::kULBeginning );
    void            flush( uint32 segmentIndex );
    void            flush();

    bool            writeUncompressed( const ULModuleIdentifier& moduleID );
};

#endif

