/**
 * @file ulditerator.h
 * @brief The interface for the ULDIterator class.
 * @author Jeff Ondich
 */

#ifndef ULDITERATOR_H
#define ULDITERATOR_H

#include "ulterm.h"

/**
 * @class ULDIterator
 * @brief An iterator that points to a particular term in a ULD module.
 */

class ULD;

class ULDIterator
{
    friend class ULD;
    friend class ULTermIdentifier;
    friend bool operator==( const ULDIterator& a, const ULDIterator& b );
    friend bool operator!=( const ULDIterator& a, const ULDIterator& b );
    friend bool operator<( const ULDIterator& a, const ULDIterator& b );

private:
    ULD             *mULD;
    ULTerm          mTerm;

    // "Position" refers to the byte offset in the Headword segment.
    // mMaxPosition represents the "end" state.  The caller should never
    // dereference the iterator via operator* if mPosition >= mMaxPosition
    // (i.e. if isAtEnd() is true).
    uint32          mPosition;
    uint32          mMinPosition;
    uint32          mMaxPosition;

    // Term offset is a byte offset within the Terms segment, pointing
    // to the current term.  mMinTermOffset is the offset stored with the
    // headword in the Headword segment, and marks the lower boundary of
    // the collection of terms associated with the headword.
    uint32          mTermOffset;
    uint32          mMinTermOffset;

    void            clear();
    void            synchronize();

public:
                    ULDIterator();
                    ULDIterator( const ULDIterator& it );
                    ~ULDIterator();

    ULDIterator&    operator=( const ULDIterator& it );
    const ULTerm&   operator*() const;
    ULDIterator&    operator++();
    ULDIterator     operator++( int );
    ULDIterator&    operator--();
    ULDIterator     operator--( int );

    void            stepForward();
    void            goToEnd();
    void            goToBeginning();
    bool            isAtEnd() const;
    bool            isAtBeginning() const;
    bool            isAtFirstTermOfHeadword() const;
    bool            isAtLastTermOfHeadword() const;
    void            move( uint32 positionNumerator, uint32 positionDenominator );
    void            goTo( const ULTerm& term );
    ULD             *getULD() { return mULD; }
    const ULD       *getULD() const { return mULD; }
    uint32          getMaxPosition() const { return mMaxPosition; }
    uint32          getMinPosition() const { return mMinPosition; }
    uint32          getPosition() const { return mPosition; }
    uint32          getMinTermOffset() const { return mMinTermOffset; }
    uint32          getTermOffset() const { return mTermOffset; }
    void            set( uint32 min, uint32 max, uint32 pos, ULLanguage lang );
    ULLanguage      getLanguage();
};

bool operator==( const ULDIterator& a, const ULDIterator& b );
bool operator!=( const ULDIterator& a, const ULDIterator& b );
bool operator<( const ULDIterator& a, const ULDIterator& b );

#endif
