/**
 * @file ulfile.h
 * @brief The interface for the <code>ULFile</code> class.
 * @author Jeff Ondich
 */

#ifndef ULFILE_H
#define ULFILE_H

#include "ulstring.h"
#include "ulmodules.h"

#ifdef UL_SYMBIAN
#include "f32file.h"
#endif

#ifdef UL_PALMOS
const uint32 kULPalmCacheCapacity = 8;
const uint32 kULPalmBlockSize = 1024;

struct ULPalmCacheEntry
{
    uint16        mRecordIndex;
    uint32        mRecordSize;
    MemHandle    mRecordHandle;
    uint8        *mRecordPtr;

                ULPalmCacheEntry();
};
#endif


/**
 * @class ULFile
 * @brief A class used to encapsulate the peculiarities of
 * a particular data source (e.g. ANSI C++ fstreams or Palm
 * OS databases) using a simple file-like interface.
 * 
 * <code>ULFile</code> would normally be an abstract class, but we
 * need to avoid the use of inheritance on Palm OS.  Therefore,
 * there are different implementations of <code>ULFile</code>
 * for different data sources.  To use <code>ULFile</code> in a
 * project, first #define the appropriate constant
 * (e.g. <code>UL_ANSI</code> or <code>UL_PALMOS</code>),
 * and then include the appropriate implmentation file
 * (e.g. ulfile-ansi.cpp or ulfile-palm.cpp) in your project.
 * The correct header file will be #included automatically.
 *
 * <code>ULFile</code> has limited write capabilities, to allow
 * only for what <code>ULD</code> needs.  On opening for writing,
 * the file is either created if it doesn't already exist, or
 * truncated to size 0 if it does.  Seeks beyond the current file
 * size are allowed, however.
 */

class ULFile
{
    uint32          mOpenMode;

private:
#if defined(UL_PALMOS)
    ULDBInfo        mDBInfo;
    uint32          mFilePosition;
    ULPalmCacheEntry mCache[kULPalmCacheCapacity];
    uint32          mCacheSize;
    uint32          mFileSize;

    void            closeRecord( uint16 cacheIndex );
    bool            loadRecord( uint16 recordIndex );

#elif defined(UL_WINDOWSMOBILE)
    ulstring        mFilePath;
    HANDLE          mFileHandle;

#elif defined(UL_SYMBIAN)
    ulstring        mFilePath;
    RFs             mFileServer;
    RFile           mFile;
    TBool           mIsOpen;

#elif !defined(UL_NOSTL)
    ulstring        mFilePath;
    fstream         mFile;
#endif


public:
    // Open modes.
    enum{ kULReadOnly=1, kULWriteOnly = 2, kULAppend = 3 };

    // Seek starting points.
    enum{ kULBeginning=1, kULCurrentPosition=2, kULEnd=3 };

                    ULFile();
                    ~ULFile();

    uint32          getOpenMode() const { return mOpenMode; }
    bool            open( const ULModuleIdentifier& moduleID, uint32 openMode );
    void            close();
    void            clear();
    bool            isOpen();
    uint32          read( uint8 *buffer, uint32 nBytes );
    bool            write( const uint8 *buffer, uint32 nBytes );
    uint32          seek( int32 offset, int origin=kULBeginning );
    uint32          tellp();
    uint32          tellg();
};

#endif

