/**
 * @file ulsimplerenderer.h
 * @brief The interface for the <code>ULSimpleRenderer</code> class.
 * @author Jeff Ondich
 */

#ifndef ULSIMPLERENDERER_H
#define ULSIMPLERENDERER_H

#include "ulrenderer.h"

/**
 * @class ULSearchContext
 * @brief A struct that encapsulates a search language, translation
 * language, and product into a single object.
 *
 * As of this writing (25 Aug 2008), we have many ULAPI functions that
 * accept search language, translation language, and product as three
 * separate parameters.  This is a hassle, and obscures the fact that
 * these three things always go together.  Eventually, we want to fix
 * this.  We're going to start here in the <code>ULRenderer</code>
 * hierarchy, and eventually move it more widely into the ULAPI interfaces.
 */
class ULSearchContext
{
private:
    ULLanguage mSearchLanguage;
    ULLanguage mTranslationLanguage;
    uint32 mProduct;

public:
    ULSearchContext();
    ULSearchContext( ULLanguage searchLang, ULLanguage translationLang, uint32 product );
    ULSearchContext( const ULSearchContext& context );
    virtual ~ULSearchContext();

    virtual void clear();
    virtual ULSearchContext& operator=( const ULSearchContext& context );
    void set( ULLanguage searchLang, ULLanguage translationLang, uint32 product );

    ULLanguage getSearchLanguage() const;
    void setSearchLanguage( ULLanguage lang );
    ULLanguage getTranslationLanguage() const;
    void setTranslationLanguage( ULLanguage lang );
    uint32 getProduct() const;
    void setProduct( uint32 product );
};


/**
 * @class ULRendererState
 * @brief A class that holds the state used during the toString operation.
 * An object of a ULRendererState subclass will hold everything toString needs
 * to keep track of during rendering.
 */
class ULRendererState
{
private:
    const ULDMIterator *mULDMItPtr;
    ULSearchContext mSearchContext;
    ULSearchContext mReverseSearchContext;
    bool mUseReverseSearchContext;
    ULListConstIterator<ULDefinition> mCurrentDefIt;

public:
    ULRendererState();
    ULRendererState( const ULRendererState& state );
    ULRendererState( const ULDMIterator *uldmItPtr );
    virtual ~ULRendererState();

    virtual void clear();
    virtual ULRendererState& operator=( const ULRendererState& state );

    void setIterator( const ULDMIterator *uldmItPtr );
    bool hasIterator() const;
    const ULTerm& getTerm() const;
    const ULDMIterator& getIterator() const;

    const ULSearchContext& getSearchContext() const;
    const ULSearchContext& getReverseSearchContext() const;
    const ULSearchContext& getCurrentSearchContext() const;

    bool hasUseReverseSearchContext() const; 
    void setUseReverseSearchContext( bool b );
    ULLanguage getSearchLanguage() const;
    void setSearchLanguage( ULLanguage lang );
    ULLanguage getTranslationLanguage() const;
    void setTranslationLanguage( ULLanguage lang );
    uint32 getProduct() const;
    void setProduct( uint32 product );

    void resetDefinition();
    bool hasDefinition() const;
    void nextDefinition();
    const ULDefinition& getDefinition() const;
};

/**
 * @struct ULRendererFeature
 * @brief A collection of rendering information related to a specific
 * ULTerm feature or ULDefinition feature.
 */

struct ULRendererFeature
{
    uint32 mFeatureID;
    const char *mStartMark; // UTF-8
    const char *mEndMark; // UTF-8
    uint32 mFeatureContexts;
};


/**
 * @class ULSimpleRenderer
 * @brief The child of <code>ULRenderer</code> that introduces a general
 * framework of interfaces supporting the most common and important ways
 * of rendering ULD data.
 *
 * Most renderers should be descendants of <code>ULSimpleRenderer</code>,
 * though the essential <code>toString</code> interface is provided by
 * <code>ULRenderer</code> in case there are unusual rendering
 * requirements, in which case a subclass may inherit directly from
 * <code>ULRenderer</code>.
 *
 * The implementation of <code>ULSimpleRenderer::toString</code> decomposes
 * the rendering task into subtasks like "start a term", "render features
 * that should go before the term text", "start a definition", etc.
 * The interfaces for these subtasks are given in <code>ULSimpleRenderer</code>,
 * though most of their implementations are deferred to the subclasses.
 * <code>ULSimpleRenderer</code> does, however, provide an infrastructure
 * for the rendering of features, using a <code>ULVector</code> of
 * <code>ULRendererFeature</code> objects.  Subclasses should initialize
 * mRendererFeatures with entries for each feature that can appear in the
 * data being rendered.  See <code>ULAppRenderer</code> for an example.
 *
 * <code>ULSimpleRenderer</code> also has a facility for maintaining
 * a table of "rendering-support" strings.  For example, the Ultralingua
 * data output includes "Phrases", "Expressions", "Alternate spelling", etc.
 * By providing an interface for clearing and setting these strings,
 * <code>ULSimpleRenderer</code> makes it straight-forward for the client
 * code to localize these strings at run-time.
 */

class ULSimpleRenderer : public ULRenderer
{
public:
    ULSimpleRenderer();
    ULSimpleRenderer( const ULSimpleRenderer& renderer );
    virtual ~ULSimpleRenderer();

    virtual void clear();
    virtual ULSimpleRenderer& operator=( const ULSimpleRenderer& renderer );

    // Rendering support strings.
    virtual void clearStrings();
    virtual bool getString( const ulstring& key, ulstring& value ) const;
    virtual void setString( const ulstring& key, const ulstring& value );

    // Feature contexts
    enum ULFeatureContext
    {
        kULNoContext = 0,
        kULBeforeTermText = 1,
        kULBeforeTermPOSP = (1 << 1),
        kULBeforeDefinitions = (1 << 2),
        kULBeforeDefinitionText = (1 << 3),
        kULBeforeDefinitionPOSP = (1 << 4),
        kULAfterDefinitionPOSP = (1 << 5),

        kULGenericContext = (kULBeforeDefinitions | kULAfterDefinitionPOSP)
    };

    virtual ulstring toString( const ULDMIterator& it );

protected:
    ULVector<ULRendererFeature> mRendererFeatures;
    enum { kULStringTableSize = 50 };
    ULHashTable<ulstring, ulstring> mStringTable;

    virtual ULRendererState *createRendererState( const ULDMIterator& uldmIt );

    virtual void startTerm( ULRendererState *state, ulstring& resultString ) = 0;
    virtual void endTerm( ULRendererState *state, ulstring& resultString ) = 0;
    virtual void processTermText( ULRendererState *state, ulstring& resultString ) = 0;
    virtual void processTermPOSP( ULRendererState *state, ulstring& resultString ) = 0;
    virtual void startDefinitions( ULRendererState *state, ulstring& resultString ) = 0;
    virtual void endDefinitions( ULRendererState *state, ulstring& resultString ) = 0;
    virtual void startDefinition( ULRendererState *state, ulstring& resultString ) = 0;
    virtual void endDefinition( ULRendererState *state, ulstring& resultString ) = 0;
    virtual void processDefinitionText( ULRendererState *state, ulstring& resultString ) = 0;
    virtual void processDefinitionPOSP( ULRendererState *state, ulstring& resultString ) = 0;

    virtual void processFeatureContext( const ULList< ULPair<ulstring, uint32> >& features,
                                        ULFeatureContext featureContext,
                                        ULRendererState *state,
                                        ulstring& resultString );

    virtual void processFeature( uint32 featureID,
                                 const ulstring& featureValue,
                                 ULFeatureContext featureContext,
                                 ULRendererState *state,
                                 ulstring& resultString );

    virtual bool hasFeatureContext( uint32 featureID, ULFeatureContext featureContext );
    virtual void startFeature( uint32 featureID, ulstring& resultString );
    virtual void endFeature( uint32 featureID, ulstring& resultString );
};

#endif

