/**
 * @file ulterm.h
 * @brief The interface for the <code>ULTerm</code> class.
 * @author Jeff Ondich
 */

#ifndef ULTERM_H
#define ULTERM_H

#include "ulstring.h"
#include "ulcontainers.h"

/**
 * Features types are stored as one-byte markers in the
 * Term segment of the ULD file format.  In the Term segment,
 * various strings, records, and sub-records are delimited using
 * the \0, \001, \002, and \003 bytes.  Thus, it's important
 * to keep those four delimiters out of the actual data.  We
 * do that by starting the feature types at 5 (leaving room
 * for \004 as a delimiter if it's needed later).
 *
 * The feature types are separated into various categories,
 * but it's really up to the processing code to decide how to
 * treat different features.
 *
 * Make sure the feature list used by the .uld builder (uldmaker.py)
 * matches this feature list.  The builder's feature list is found in
 * ulutilities.py.
 */
enum
{
    // "Features" that do not appear in the .uld file format, but that
    // are used by ULDManager::getSemanticMarkup.
    kULDummyFeature=0,
    kULTermTextFeature,
    kULHeadwordFeature,
    kULRegexSelectionFeature,
    kULPartOfSpeechFeature,
    kULDefPartOfSpeechFeature,
    kULUserAdditionFeature,

    // Special features, UL data.
    kULOrder,
    kULDefinition,
    kULIndex,
    kULType, /* 10 */
    kULKeyWord,
    kULCategory,

    // Term features, UL data.
    kULPronunciation,
    kULUsage,
    kULEtymology,
    kULHomonym,
    kULSynonym,
    kULAntonym,
    kULAlternateSpelling,
    kULForm, /* 20 */

    // Definition features, UL data.
    kULDefForm,
    kULPreposition,
    kULDefPreposition,
    kULContext,
    kULDefContext,
    kULTone,
    kULDefTone,
    kULDomain,
    kULDefDomain,
    kULDialect, /* 30 */
    kULDefDialect,
    
    kULNUltralinguaFeatureTypes,

    // Collins features.
    kULBox = kULNUltralinguaFeatureTypes,
    kULAbbreviation,
    kULAltForm,
    kULPhrasalVerb,
    kULFirstTermOfHeadword,
    kULKeyword,
    kULAddOn,
    kULSubstitute, 
    kULInflection, /* 40 */
    kULInflectionAltForm,
    kULInflectionSubstitute,

    kULFirstLabel,
    kULLabelGrammar=kULFirstLabel,
    kULLabelSyntax,
    kULLabelModifier,
    kULLabelSyntaxAlso,
    kULLabelOpenCollocate,
    kULLabelOpenNN,
    kULLabelOpenObject,
    kULLabelOpenSubject, /* 50 */
    kULLabelOpenVerb,
    kULLabelOpenFullForm,
    kULLabelOpenRagbag,
    kULLabelOpenSuper,
    kULLabelOpenSyn,
    kULLabelRegional,
    kULLabelRegister,
    kULLabelSubjectField,
    kULLabelTrademark, /* 60 */
    kULLabelOpenAdditionalGrammar,
    kULLabelOpenGenitiveAndPlural,
    kULLabelOpenAuthor,
    kULLabelOpenSource,
    kULLabelOpenGrammar,
    kULLabelOpenGrammarWord,
    kULLabelOpenGrammarIntroduction,
    kULLabelOpenGrammarRagbag,
    kULLabelPOSP,
    kULLastLabel=kULLabelPOSP,

    kULStartPhraseBlock,
    kULEndPhraseBlock, /* 70 */
    kULStartPhrase,
    kULEndPhrase,
    kULOptPrep,
    kULNumber,
    kULPhraseNumber,
    kULExpansion,
    kULTargetEquivalent,
    kULGloss,
    kULSourceEquivalent,
    kULSeeAlso, /* 80 */
    kULSee,
    kULXRef,
    kULXRLK,
    kULSentenceLexicalized,
    kULInflectionPronunciation,
    kULPhrase,
    kULExamplePhrase,
    kULLike,
    kULShortFor,
    kULAlso, /* 90 */
    kULSymbol,
    kULVariantForm,
    kULAlsoCalled,
    kULFormalName,
    kULSameAs,
    kULRelatedAdj,
    kULAnotherWordFor,
    kULPastTenseOf,
    kULKeySynonym,
    kULKeyAntonym, /* 100 */
    kULQuotation,
    kULProverb,
    kULRelatedWord,
    kULWebLink,
    kULThesaurusDefinition,
    kULLexicalSetStructure,
    kULSeeNote,
    kULRefUseLabelText,
    kULWithEtc,
    kULSubsemantic, /* 110 */
    kULFeminineForm,
    kULDerivation,
    kULMinimalUnit,
    kULSecondEquiv,
    kULRunOn,
    kULSeeBox,
    kULEmbeddedLabel,
    kULSpellingReform,
    kULLabelLanguage,
    kULLabelSubjectFieldUnderlying, /* 120 */
    kULSubentry,
    kULSense,
    kULIdiom,
    kULFirstSynonym,

    kULBiography,
    kULSurname,
    kULGivenName,
    kULBirthDate,
    kULDeathDate,
    kULGender, /* 130 */
    kULNationality,
    kULBirthNationality,
    kULOccupationPrimary,
    kULOccupationSecondary,
    kULOccupationTertiary,
    kULOccupationFourth,

    kULPOSPModifier,
    kULExtraPOSP,
	kULGrammar,

    // 4/20/08: COBUILD Advanced special features.
    kULImagePictureDictionary,
    kULImageWordWeb, /* 140 */
    kULWordset,
    kULCollinsThesaurus,
    kULWordPartnership,
    kULWordLink,
    kULLabelFreqBand,
    kULBoxType,

    kULGenericPhrase,
    kULEndGenericPhrase,
    kULReflexiveVerb,
    kULTranslationInflection,
    
    // 12/1/2008: Collins tagged features.
    kULHeadwordGrammar,
    kULHeadwordEnding,
    kULHeadwordCompleteForm,
    kULHeadwordGenitive,
    kULHeadwordHomonymNumber,
    kULHeadwordRegionalVariant,
    kULInflectionsGrammar,
    kULHeadwordPluralForm,
    kULPhraseIdiom,
    kULReflexiveExtension,
    kULTranslationEquivalent,
    kULCompound,
    kULEndDefinitionGrouping,
    kULPinyin,
    kULTTS,

    // 7/11/2009: Vox features
    kULObservation,
    kULSubdefinition,

    kULNFeatureTypes
};

// Utility functions for ULTerm::toXHTML().
void startV7Feature( ulstring& s, uint32 feature );
void endV7Feature( ulstring& s, uint32 feature );

/**
 * @class ULDefinition
 * @brief The class used to represent a dictionary definition and its features.
 *
 * <code>ULDefinition</code> objects are used inside <code>ULTerm</code> objects
 * to represent the definitions/translations of the term.  For example, if the
 * term represents "jump v." in an English-French dictionary, then the definitions
 * might be "sauter v.", "faire un saut expr.", "tressaillir v. (~ with surprise)",
 * "tressauter v. (with surprise, etc.)", and "sursauter v. (with surprise, etc.)".
 * The annotations in these definitions ("~ with surprise", etc.) are stored as
 * "features" in the .uld files, and get added to the <code>mFeatures</code> list
 * when the definition is loaded from a file.
 *
 * The <code>mPriority</code> is the priority number associated with the .uld file
 * from which the definition comes, and is used to help sort definitions when two
 * .uld files are being merged at runtime.
 */
struct ULDefinition
{
public:
    ulstring                            mText;
    ulstring                            mPartOfSpeech;
    uint32                              mPartOfSpeechRank;
    uint32                              mPriority;
    ULList< ULPair<ulstring,uint32> >   mFeatures;

                                        ULDefinition();
                                        ULDefinition( const ULDefinition& def );
                                        ~ULDefinition();

    ULDefinition&                       operator=( const ULDefinition& def );
    void                                clear();
    bool                                getFeature( uint32 type, ulstring& value ) const;
    void                                addFeature( uint32 type, const ulstring& value );
    void                                merge( const ULDefinition& def );
};


/**
 * @class ULTerm
 * @brief The class used to represent a dictionary term and its features.
 *
 * <code>ULTerm</code> objects are used to store data extracted from a .uld
 * file.  Each <code>ULTerm</code> object represents a single term, all its
 * definitions/translations, and any features associated with either the term
 * as a whole or a particular definition.
 */
struct UL_API ULTerm
{
public:
    ULLanguage                          mLanguage;
    ULLanguage                          mTranslationLanguage;
    uint32                              mProduct;
    ulstring                            mHeadword;
    ulstring                            mText;
    ulstring                            mPartOfSpeech;
    uint32                              mPartOfSpeechRank;
    ULList< ULPair<ulstring,uint32> >   mFeatures;
    ULList<ULDefinition>                mDefinitions;

                                        ULTerm();
                                        ULTerm( const ULTerm& entry );
                                        ~ULTerm();

    ULTerm&                             operator=( const ULTerm& entry );
    void                                clear();
    bool                                getFeature( uint32 type, ulstring& value ) const;
    void                                addFeature( uint32 type, const ulstring& value );
    bool                                canBeMergedWith( const ULTerm& term ) const;
    void                                merge( const ULTerm& term );
    void                                print() const;
    uint32                              hash( uint32 tableSize ) const;
};

UL_API bool operator==( const ULTerm& a, const ULTerm& b );
UL_API bool operator!=( const ULTerm& a, const ULTerm& b );
UL_API bool operator<( const ULTerm& a, const ULTerm& b );
UL_API bool operator<=( const ULTerm& a, const ULTerm& b );

#endif
