/**
 * @file uld.h
 * @brief The interface for the <code>ULD</code>, <code>ULDIterator</code>,
 * and <code>ULDHeader</code> classes.
 * @author Jeff Ondich
 */

#ifndef ULD_H
#define ULD_H

#include "ulditerator.h"
#include "ulcompressor.h"
#include "ultokenizer.h"
#include "ulmodules.h"
#include "ulsegmentedfile.h"


enum
{
    kULDFirstSegment = 0,
    kULDHeaderSegment = kULDFirstSegment,
    kULDPartOfSpeechSegment,
    kULDHeadwordSegment,
    kULDIntegritySegment,
    kULDTermSegment,

    kULDNRequiredSegments,

    kULDRedPrefixSegment = kULDNRequiredSegments,
    kULDGreenPrefixSegment,

    kULDNSegments
};


// Used for .uld integrity checking.
#define uldsmoosh(a,b) (uint32((67*(a))+(23*(b))))

// For part-of-speech management.
const uint32 kULPartOfSpeechIndexBase = 4;

// For identification of modifiable definitions.
const uint32 kULDAdditionsPriority = 100;

/**
 * @class ULDHeader
 * @brief The in-memory representation of the ULD header found at
 * the top of the .uld file format (version 5.4).
 */

struct ULDHeader
{
public:
    uint32                  mMagicNumber;
    uint32                  mFileSize;
    uint32                  mVersion;
    ULLanguage              mRedLanguage;
    ULLanguage              mGreenLanguage;

    uint32                  mNRedHeadwords;
    uint32                  mNGreenHeadwords;
    uint32                  mGreenHeadwordStart;

    uint32                  mNPartsOfSpeech;
    uint32                  mPriority;
    uint32                  mRuntimePriority;
    uint32                  mIntegrityCheck;

                            ULDHeader();
    void                    clear();
};


/**
 * @class ULD
 * @brief Each <code>ULD</code> object represents a single dictionary file,
 * and supports searching, adding, and deleting.
 */

class ULD
{
    friend class ULDIterator;

public:
                        ULD();
                        ULD( ULCompressor *compressor, ULTokenizer *tokenizer );
                        ~ULD();

    void                setCompressor( ULCompressor *compressor );
    void                setTokenizer( ULTokenizer *tokenizer );

    bool                attach( const ULModuleIdentifier& moduleID );
    void                detach();
    bool                open( uint32 openMode=ULFile::kULReadOnly );
    bool                open( const ULModuleIdentifier& moduleID, uint32 openMode=ULFile::kULReadOnly );
    void                close();
    bool                isOpen() const;

    uint32              getVersion() const { return (mHeader.mVersion & kULVersionMask); }
    uint32              getPlatform() const { return UL_PLATFORM(mHeader.mVersion); }
    uint32              getProduct() const { return UL_PRODUCT(mHeader.mVersion); }
    uint32              getSize() const { return mHeader.mFileSize; }
    const ULModuleIdentifier& getModuleIdentifier() const;

    uint32              getPriority() const { return mHeader.mPriority; }
    void                setRuntimePriority( uint32 priority ) { mHeader.mRuntimePriority = priority; }

    ULLanguage          getRedLanguage() const;
    ULLanguage          getGreenLanguage() const;
    ULLanguage          getOtherLanguage( ULLanguage lang ) const;
    bool                isThesaurus() const;
    bool                isMonolingual() const;
    bool                isModifiable() const;

    void                begin( ULLanguage lang, ULDIterator& it );
    void                end( ULLanguage lang, ULDIterator& it );

    uint32              getPartOfSpeechRank( const ulstring& posp ) const;
    void                getAdditionsPartsOfSpeech( ULList< ULPair<ulstring,uint32> >& pospList, ULLanguage lang ) const;

    void                find( const ulstring& searchString, ULLanguage lang, ULDIterator& it );
    void                find( const ULTerm& searchTerm, ULDIterator& it );
    void                getApproximateMatches( const ulstring& searchString,
                                               ULLanguage searchLanguage,
                                               uint32 requiredPrefixLength,
                                               ULList<ulstring>& headwordList );

    void                addTerm( const ULTerm& term );
    void                addTerms( const ULList<ULTerm>& newTerms );
    bool                deleteDefinition( const ULTerm& term );

    static bool         createEmptyULD( ULModuleIdentifier& moduleID,
                                        ULLanguage lang1,
                                        ULLanguage lang2,
                                        uint32 product,
                                        ULCompressor *compressor ); // platform-dependent

    bool                writeUncompressed( const ULModuleIdentifier& moduleID );

private:
    ULSegmentedFile     mSegmentedFile;
    ULModuleIdentifier  mModuleIdentifier;
    ULDHeader           mHeader;
    ULTokenizer         *mTokenizer;

    enum { kULPOSPRankTableSize = 397 };
    ulstring                        *mPartsOfSpeech;
    ULHashTable<ulstring,uint32>    mPartOfSpeechRanks;

    bool                bsearch( const ulstring& searchHeadword,
                                 ULLanguage lang,
                                 ulstring& foundHeadword,
                                 uint32& minPosition,
                                 uint32& maxPosition,
                                 uint32& position );

    enum { kULMaxHeadwordLength = 16, kULInsertionCost = 1, kULDeletionCost = 1, kULSubstitutionCost = 2, kULMaxAcceptableEditDistance = 2 };
    int16               mDistanceTable[kULMaxHeadwordLength*kULMaxHeadwordLength];
    int16               getEditDistance( ulstring& s, ulstring& t );

    void                loadPartsOfSpeech();
    void                getTerm( ULTerm& term, uint32 position, uint32 termOffset );

    void                loadAllTerms( ULList<ULTerm>& termList );
    void                writeTermsToFile( ULList<ULTerm>& termList,
                                          ULList< ULPair<ulstring,uint32> >& partsOfSpeech,
                                          ULHashTable<ulstring,uint32>& pospToIndex );
};


#endif
