/**
 * @file uldmanager.h
 * @brief The interface for the <code>ULDManager</code> class.
 * @author Jeff Ondich
 */

#ifndef ULDMANAGER_H
#define ULDMANAGER_H

#include "ulstring.h"
#include "ulcontainers.h"
#include "uld.h"
#include "uldmiterator.h"
#include "ulkeys.h"
#include "ultokenizer.h"


////////////////////////////////////////////////////
// ULDManager
////////////////////////////////////////////////////

class ULDManager
{
    friend class ULDMIterator;
    friend class ULTermIdentifier;

private:
    ULList<ULD *>   mULDList;
    ULRegistrar     *mRegistrar;
    ULTokenizer     *mTokenizer;
    const char      **mCollinsFormatStrings;
    const char      **mULCommonFormatStrings;
    const char      **mCollinsCommonFormatStrings;

    static void     appendMarkedWords( ulstring& mainString,
                                       const ulstring& newString,
                                       uint32 options,
                                       const ulstring& startMark,
                                       const ulstring& endMark );

    static void     insertWordMarks( ulstring& s, const ulstring& startMark, const ulstring& endMark );
    static void     startFeature( ulstring& s, uint32 feature );
    static void     endFeature( ulstring& s, uint32 feature );
    bool            needFeatureMarks( uint32 feature, uint32 product );
    int             mergeTermAndPOSP( bool isFirstTerm, const ulstring& term, const ulstring& posp, ulstring& s );
    void            getPronunciationString( const ulstring& rawPronString, uint32 product, ulstring& pronString );
    void            getCollinsSemanticMarkup( const ULDMIterator& it, ulstring &markedString, uint32 options=0 );
    void            getULSemanticMarkup( const ULDMIterator& it, ulstring &markedString, uint32 options=0 );

public:
    static ULDManager *create();

                    ULDManager();
                    ~ULDManager();

    static bool     isLabel( uint32 feature );

    bool            init();

    uint32          getNDictionaries();
    ULD             *getDictionary( uint32 n );

    void            addDictionary( ULD *uld );
    ULD             *deleteDictionary( uint32 n );

    void            closeDictionary( ULLanguage lang1, ULLanguage lang2, uint32 product );
    void            closeAllDictionaries();

    void            setRegistrar( ULRegistrar *registrar ) { mRegistrar = registrar; }
    void            setTokenizer( ULTokenizer *tokenizer ) { mTokenizer = tokenizer; }

    enum { kULGFSAddWordMarks=1, kULGFSStemmedDisplay=2, kULGFSRegexDisplay=4, kULGFS_TTSEnabled=8, kULGFS_CollinsWeb=16 };
    void            getSemanticMarkup( const ULDMIterator& it, ulstring &markedString, uint32 options=0 );
    void            getFormattedString( const ULDMIterator& it, ulstring &formattedString, uint32 options=0 );

    // Returns an iterator pointing to the beginning or end of the red or green
    // half of the dictionary, depending on the value of lang.
    void            begin( ULLanguage searchLanguage,
                           ULLanguage translationLanguage,
                           uint32 product,
                           ULDMIterator& it );

    void            end( ULLanguage searchLanguage,
                         ULLanguage translationLanguage,
                         uint32 product,
                         ULDMIterator& it );

    // Returns an iterator pointing to the best match for searchString
    // in the half of the dictionary specified by lang.
    void            find( const ulstring& searchString,
                          ULLanguage searchLanguage,
                          ULLanguage translationLanguage,
                          uint32 product,
                          ULDMIterator& it );

    void            find( const ULTerm& searchTerm, uint32 product, ULDMIterator& it );

#ifdef UL_REGEX
    void            initRegex( const ulstring& searchString,
                               ULLanguage searchLanguage,
                               ULLanguage translationLanguage,
                               uint32 product,
                               const ULDMRegexOptions& options,
                               ULDMIterator& it );
#endif
};

#endif
