/**
 * @file ultermidentifier.h
 * @brief The interface for the <code>ULTermIdentifier</code> class.
 * @author Jeff Ondich
 */

#ifndef ULTERMIDENTIFIER_H
#define ULTERMIDENTIFIER_H

#include "uldmiterator.h"

/**
 * @class ULTermIdentifier
 * @brief Represents a <code>ULDMIterator</code> object as a sequence of integers.
 *   The point of doing this is that we can store the integers in question as
 *   attributes of a &lt;div&gt; in some XHTML code, and then reconstruct the
 *   corresponding <code>ULDMIterator</code> in reaction to some Javascript event.
 *
 * Watch out, though.  Any given <code>ULTermIdentifier</code> object is dependent
 * for its coherence on the <code>ULDManager</code> object into which its
 * corresponding <code>ULDMIterator</code> object pointed.  So if you call
 * <code>addDictionary</code> or <code>deleteDictionary</code> on the
 * <code>ULDManager</code> after obtaining a <code>ULTermIdentifier</code> from
 * a <code>ULDMIterator</code>, then both the <code>ULTermIdentifier</code> and
 * the <code>ULDMIterator</code> objects become invalid.
 *
 * Generally, you don't need to know the internal structure of the
 * <code>ULTermIdentifier</code>, since you can just get it from the
 * <code>ULDMIterator</code>, and then rebuild the <code>ULDMIterator</code>
 * later using <code>ULTermIdentifier::getIterator</code>.  If you want to
 * know the details, however, here they are.
 *
 * The <code>ULTermIdentifier</code> consists of a pointer to the
 * <code>ULDManager</code> object from which it came, plus a sequence of unsigned
 * integers.  The sequence of integers starts with the three integers that
 * identify the dictionary modules to which the identifier refers:
 *
 * <ul>
 *     <li>Search language</li>
 *     <li>Translation language</li>
 *     <li>Product</li>
 * </ul>
 *
 * Then, for each uld file managed by the <code>ULDManager</code> that matches
 * the (language, language, product) triple, we have an additional five integers
 * (see documentation of <code>ULDIterator</code> for an explanation of these
 * integers).
 *
 * <ul>
 *     <li>Minimum position</li>
 *     <li>Maximum position</li>
 *     <li>Position</li>
 *     <li>Minimum term offset</li>
 *     <li>Term offset</li>
 * </ul>
 */

class ULDManager;
class ULDMIterator;

class ULTermIdentifier
{
    friend class ULDManager;
    friend class ULDMIterator;

public:
    ULTermIdentifier();
    ULTermIdentifier( const ULTermIdentifier& it );
    ULTermIdentifier( const ULList<uint32>& intList, ULDManager *manager );
    ~ULTermIdentifier();

    ULTermIdentifier& operator=( const ULTermIdentifier& it );
    void clear();
    void getIterator( ULDMIterator& uldmIterator );
	ULList<uint32> getAddressInformation() const;
	
private:
    ULDManager *mULDManager;
    ULList<uint32> mIntegerList;
};

#endif
