///////////////////////////////////////////////////////////////////
//
//    ulkeys.h
//
//    A typical registration key looks like this:
//
//        M4EFM425502819368-JEFFONDICH
//
//    The platform, program version, language pair, optional version
//    modifier like "M" for "medical", and user name are all presented in
//    human-readable form.  The numerical string at the beginning
//    is called the "code" in the source code below.  The code is
//    an encrypted string that contains the platform, license size, 
//    program version, language pair, and version modifiers.  The code
//    is encrypted using an encryption key derived from the user
//    name.  Therefore, any change to the human-readable portion
//    of the registration key is going to yield an invalid key.
//
//    The key is insensitive to case, punctuation, and diacritical
//    marks in the user name.  The case and punctuation in the rest
//    of the key is required.
//
//    The structure of the unencrypted code is:
//
//        platform ID            2 decimal digit
//        license size        2 digits
//        version number        1 digit
//        first language        2 digits
//        modifier            2 digits
//        serial number        4 digits
//        second language        2 digits
//
///////////////////////////////////////////////////////////////////

#ifndef ULKEYS_H
#define ULKEYS_H

#include "ulutilities.h"
#include "ulcontainers.h"


///////////////////////////////////////////////////////////////////
// ULKeyHandler is just a bunch of global functions in a box.
///////////////////////////////////////////////////////////////////

// The main data structure for storing registration information
struct UL_API ULRegistrationInfo
{
    ulstring                mKey;
    ulstring                mUserName;
    uint32                  mLicenseSize;
    uint32                  mSerialNumber; // restricted to 8 decimal digits
    uint32                  mPlatform;
    ULLanguage              mFirstLanguage;
    ULLanguage              mSecondLanguage;
    uint32                  mMajorVersionNumber;
    uint32                  mProduct;  // e.g. Medical

    uint32                  mYear;
    uint32                  mMonth;
    uint32                  mDay;
    uint32                  mNDaysUsed;
    bool                    mRegistered;
    

                            ULRegistrationInfo();
                           ~ULRegistrationInfo();

    void                    clear();
    void                    set( const ulstring& name, uint16 size, uint32 serial,
                                uint16 platformID, ULLanguage lang1, ULLanguage lang2,
                                uint16 versionNumber, uint16 modifier ) ;

    bool                    operator<( const ULRegistrationInfo& info ) const;
    bool                    operator==( const ULRegistrationInfo& info ) const;
};


class ULKeyHandler
{
public:
    static bool         keyToInfo( const ulstring& key, ULRegistrationInfo& regInfo );
    static void         infoToKey( const ULRegistrationInfo& regInfo, ulstring& key ) ;

    static ULLanguage   getLanguage( const ulstring_iterator& it );
    static ulchar       getLanguageInitial( ULLanguage lang ) ;
    static uint32       getPlatform( const ulstring_iterator& it );
    static ulchar       getPlatformInitial( uint32 platformID ) ;
    static uint32       getProduct( const ulstring_iterator& it );
    static ulchar       getProductInitial( uint32 modifier ) ;
    static uint32       getEncryptionKey( const ULRegistrationInfo& regInfo );
    static uint32       getInt( ulstring_iterator& it, uint32 nDigits );

    // Applicable only to digit strings
    static void         encryptDigits( ulstring& s, uint32 encryptionKey ) ;
    static void         decryptDigits( ulstring& s, uint32 encryptionKey );
    static void         rotateDigits( ulstring& s, uint32 rotation, uint32 boost );
    static void         ninesComplement( ulstring& s );
    static void         zipDigits( ulstring& s, uint32 boost, bool forward=true );
    static void         unzipDigits( ulstring& s, uint32 boost );

    // Applicable to any strings
    static void         rotate( ulstring& s, uint32 rotation ) ;
    static void         twist( ulstring& s );
    static void         shear( ulstring& s, bool forward, ulchar boost=ulchar('0') ) ;
    static void         zip( ulstring& s, bool forward ) ;
    static void         unzip( ulstring& s, bool forward );
    static void         ascify( ulstring& s ) ;
    static void         deascify( ulstring& s );
};


class ULRegistrar
{
private:
    ULList<ULRegistrationInfo>  mRegInfoList;
    uint32                      mNDaysInTrialPeriod;
    int                         mTrialType;

    static void         encryptStorageString( ulstring& s, uint32 encryptionKey ) ;
    static void         decryptStorageString( ulstring& s, uint32 encryptionKey );
	static uint32       daysBetweenDates( uint32 year1, uint32 month1, uint32 day1, uint32 year2, uint32 month2, uint32 day2 );

public:
	enum { kULDaysUsedTrialType=0, kULDaysElapsedTrialType, kULNTrialTypes };

    static ULRegistrar  *create();

                        ULRegistrar() ;
                        ~ULRegistrar() ;

    bool                init();
    void                clear();

    bool                addKeyStorageString( const ulstring& s ) ;
 
    void                setTrialPeriod( uint32 daysInTrial ) { mNDaysInTrialPeriod = daysInTrial; }
    uint32              getTrialPeriod() const { return mNDaysInTrialPeriod; }
    void                setTrialType( int trialType ) { mTrialType = trialType; }
    int                 getTrialType() const { return mTrialType; }

    uint32              getPlatform() const;
    void                getKeyStorageStringList( ULList<ulstring>& kssList );
    void                getKeyList( ULList<ulstring>& keyList );

    bool                productMatchesRegInfo( ULLanguage lang1, ULLanguage lang2, uint32 product, uint32 platform, const ULRegistrationInfo& regInfo ) const;
    bool                isRegistered( ULLanguage lang1, ULLanguage lang2, uint32 platform, uint32 product ) const;
    bool                isExpired( ULLanguage lang1, ULLanguage lang2, uint32 platform, uint32 product ) const;
    uint32              daysUsed( ULLanguage lang1, ULLanguage lang2, uint32 platform, uint32 product ) const;
    ulstring            userName( ULLanguage lang1, ULLanguage lang2, uint32 platform, uint32 product ) const;


    void                useProduct( ULLanguage lang1, ULLanguage lang2, uint32 platform, uint32 product,
                                        uint32 year, uint32 month, uint32 day ) ;

    bool                submitRegistrationKey( const ulstring& key, uint32 platform, uint32 year, uint32 month, uint32 day, ULRegistrationInfo& regInfo ) ;

    void                removeRegistrationKey( const ulstring& key ) ;

    static void         createKeyStorageString( ulstring& kss, const ULRegistrationInfo& regInfo );
    static bool         parseKeyStorageString( const ulstring& kss, ULRegistrationInfo& regInfo );
    static bool         pastDropDeadDate();
};

#endif
